@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -157,18 +157,29 @@ class CommandDispatcher:
         else:
             return None
 
-    def _tab_focus_last(self, *, show_error=True):
+    def _tab_focus_stack(self, mode: str, *, show_error=True):
         """Select the tab which was last focused."""
+        tab_deque = self._tabbed_browser.tab_deque
+        cur_tab = self._cntwidget()
+
         try:
-            tab = objreg.get('last-focused-tab', scope='window',
-                             window=self._win_id)
-        except KeyError:
+            if mode == "last":
+                tab = tab_deque.last(cur_tab)
+            elif mode == "stack-prev":
+                tab = tab_deque.prev(cur_tab)
+            elif mode == "stack-next":
+                tab = tab_deque.next(cur_tab)
+            else:
+                raise NotImplementedError(
+                    "Missing implementation for stack mode!")
+        except IndexError:
             if not show_error:
                 return
-            raise cmdutils.CommandError("No last focused tab!")
+            raise cmdutils.CommandError("Could not find requested tab!")
+
         idx = self._tabbed_browser.widget.indexOf(tab)
         if idx == -1:
-            raise cmdutils.CommandError("Last focused tab vanished!")
+            raise cmdutils.CommandError("Requested tab vanished!")
         self._set_current_index(idx)
 
     def _get_selection_override(self, prev, next_, opposite):
@@ -423,6 +434,10 @@ class CommandDispatcher:
                    in which case the closest match will be taken.
             keep: If given, keep the old tab around.
         """
+        if config.val.tabs.tabs_are_windows:
+            raise cmdutils.CommandError("Can't take tabs when using "
+                                        "windows as tabs")
+
         tabbed_browser, tab = self._resolve_buffer_index(index)
 
         if tabbed_browser is self._tabbed_browser:
@@ -447,6 +462,10 @@ class CommandDispatcher:
             keep: If given, keep the old tab around.
             count: Overrides win_id (index starts at 1 for win_id=0).
         """
+        if config.val.tabs.tabs_are_windows:
+            raise cmdutils.CommandError("Can't give tabs when using "
+                                        "windows as tabs")
+
         if count is not None:
             win_id = count - 1
 
@@ -479,7 +498,7 @@ class CommandDispatcher:
         # Catch common cases before e.g. cloning tab
         if not forward and not history.can_go_back():
             raise cmdutils.CommandError("At beginning of history.")
-        elif forward and not history.can_go_forward():
+        if forward and not history.can_go_forward():
             raise cmdutils.CommandError("At end of history.")
 
         if tab or bg or window:
@@ -629,9 +648,9 @@ class CommandDispatcher:
         assert what in ['url', 'pretty-url', 'markdown'], what
         flags = QUrl.RemovePassword
         if what == 'pretty-url':
-            flags |= QUrl.DecodeReserved
+            flags |= QUrl.DecodeReserved  # type: ignore
         else:
-            flags |= QUrl.FullyEncoded
+            flags |= QUrl.FullyEncoded  # type: ignore
         url = QUrl(self._current_url())
         url_query = QUrlQuery()
         url_query_str = urlutils.query_string(url)
@@ -642,13 +661,15 @@ class CommandDispatcher:
             if key in config.val.url.yank_ignored_parameters:
                 url_query.removeQueryItem(key)
         url.setQuery(url_query)
-        return url.toString(flags)
+        return url.toString(flags)  # type: ignore
 
     @cmdutils.register(instance='command-dispatcher', scope='window')
     @cmdutils.argument('what', choices=['selection', 'url', 'pretty-url',
-                                        'title', 'domain', 'markdown'])
-    def yank(self, what='url', sel=False, keep=False, quiet=False):
-        """Yank something to the clipboard or primary selection.
+                                        'title', 'domain', 'markdown',
+                                        'inline'])
+    def yank(self, what='url', inline=None,
+             sel=False, keep=False, quiet=False):
+        """Yank (copy) something to the clipboard or primary selection.
 
         Args:
             what: What to yank.
@@ -658,13 +679,20 @@ class CommandDispatcher:
                 - `title`: The current page's title.
                 - `domain`: The current scheme, domain, and port number.
                 - `selection`: The selection under the cursor.
-                - `markdown`: Yank title and URL in markdown format.
+                - `markdown`: Yank title and URL in markdown format
+                  (deprecated, use `:yank inline [{title}]({url})` instead).
+                - `inline`: Yank the text contained in the 'inline' argument.
 
             sel: Use the primary selection instead of the clipboard.
             keep: Stay in visual mode after yanking the selection.
             quiet: Don't show an information message.
+            inline: A block of text, to be yanked if 'what'
+                is inline and ignored otherwise.
         """
-        if what == 'title':
+        if what == 'inline':
+            s = inline
+            what = 'inline block'
+        elif what == 'title':
             s = self._tabbed_browser.widget.page_title(self._current_index())
         elif what == 'domain':
             port = self._current_url().port()
@@ -685,6 +713,8 @@ class CommandDispatcher:
             caret.selection(callback=_selection_callback)
             return
         elif what == 'markdown':
+            message.warning(":yank markdown is deprecated, use `:yank inline "
+                            "[{title}]({url})` instead.")
             idx = self._current_index()
             title = self._tabbed_browser.widget.page_title(idx)
             url = self._yank_url(what)
@@ -828,7 +858,7 @@ class CommandDispatcher:
             idx = int(index_parts[1])
         elif len(index_parts) == 1:
             idx = int(index_parts[0])
-            active_win = objreg.get('app').activeWindow()
+            active_win = QApplication.activeWindow()
             if active_win is None:
                 # Not sure how you enter a command without an active window...
                 raise cmdutils.CommandError(
@@ -879,7 +909,7 @@ class CommandDispatcher:
         tabbed_browser.widget.setCurrentWidget(tab)
 
     @cmdutils.register(instance='command-dispatcher', scope='window')
-    @cmdutils.argument('index', choices=['last'])
+    @cmdutils.argument('index', choices=['last', 'stack-next', 'stack-prev'])
     @cmdutils.argument('count', value=cmdutils.Value.count)
     def tab_focus(self, index: typing.Union[str, int] = None,
                   count: int = None, no_last: bool = False) -> None:
@@ -890,16 +920,18 @@ class CommandDispatcher:
 
         Args:
             index: The tab index to focus, starting with 1. The special value
-                   `last` focuses the last focused tab (regardless of count).
-                   Negative indices count from the end, such that -1 is the
-                   last tab.
+                   `last` focuses the last focused tab (regardless of count),
+                   and `stack-prev`/`stack-next` traverse a stack of visited
+                   tabs. Negative indices count from the end, such that -1 is
+                   the last tab.
             count: The tab index to focus, starting with 1.
             no_last: Whether to avoid focusing last tab if already focused.
         """
         index = count if count is not None else index
 
-        if index == 'last':
-            self._tab_focus_last()
+        if index in {'last', 'stack-next', 'stack-prev'}:
+            assert isinstance(index, str)
+            self._tab_focus_stack(index)
             return
         elif index is None:
             self.tab_next()
@@ -911,7 +943,7 @@ class CommandDispatcher:
             index = self._count() + index + 1
 
         if not no_last and index == self._current_index() + 1:
-            self._tab_focus_last(show_error=False)
+            self._tab_focus_stack('last', show_error=False)
             return
 
         if 1 <= index <= self._count():
@@ -970,8 +1002,9 @@ class CommandDispatcher:
     @cmdutils.register(instance='command-dispatcher', scope='window',
                        maxsplit=0, no_replace_variables=True)
     @cmdutils.argument('count', value=cmdutils.Value.count)
+    @cmdutils.argument('output_messages', flag='m')
     def spawn(self, cmdline, userscript=False, verbose=False,
-              output=False, detach=False, count=None):
+              output=False, output_messages=False, detach=False, count=None):
         """Spawn a command in a shell.
 
         Args:
@@ -982,7 +1015,8 @@ class CommandDispatcher:
                               (or `$XDG_DATA_HOME`)
                             - `/usr/share/qutebrowser/userscripts`
             verbose: Show notifications when the command started/exited.
-            output: Whether the output should be shown in a new tab.
+            output: Show the output in a new tab.
+            output_messages: Show the output as messages.
             detach: Whether the command should be detached from qutebrowser.
             cmdline: The commandline to execute.
             count: Given to userscripts as $QUTE_COUNT.
@@ -1024,9 +1058,13 @@ class CommandDispatcher:
         else:
             cmd = os.path.expanduser(cmd)
             proc = guiprocess.GUIProcess(what='command', verbose=verbose,
+                                         output_messages=output_messages,
                                          parent=self._tabbed_browser)
             if detach:
-                proc.start_detached(cmd, args)
+                ok = proc.start_detached(cmd, args)
+                if not ok:
+                    message.info("Hint: Try without --detach for a more "
+                                 "detailed error")
             else:
                 proc.start(cmd, args)
             proc.finished.connect(_on_proc_finished)
@@ -1196,13 +1234,14 @@ class CommandDispatcher:
             objreg.get('bookmark-manager').delete(url)
         except KeyError:
             raise cmdutils.CommandError("Bookmark '{}' not found!".format(url))
+        message.info("Removed bookmark {}".format(url))
 
     @cmdutils.register(instance='command-dispatcher', name='inspector',
                        scope='window')
     def toggle_inspector(self):
         """Toggle the web inspector.
 
-        Note: Due a bug in Qt, the inspector will show incorrect request
+        Note: Due to a bug in Qt, the inspector will show incorrect request
         headers in the network tab.
         """
         tab = self._current_widget()
@@ -1238,7 +1277,6 @@ class CommandDispatcher:
             target = downloads.FileDownloadTarget(dest)
 
         tab = self._current_widget()
-        user_agent = tab.private_api.user_agent()
 
         if url:
             if mhtml_:
@@ -1246,7 +1284,7 @@ class CommandDispatcher:
                                             "page as mhtml.")
             url = urlutils.qurl_from_user_input(url)
             urlutils.raise_cmdexc_if_invalid(url)
-            download_manager.get(url, user_agent=user_agent, target=target)
+            download_manager.get(url, target=target)
         elif mhtml_:
             tab = self._current_widget()
             if tab.backend == usertypes.Backend.QtWebEngine:
@@ -1267,7 +1305,6 @@ class CommandDispatcher:
 
             download_manager.get(
                 self._current_url(),
-                user_agent=user_agent,
                 qnam=qnam,
                 target=target,
                 suggested_fn=suggested_fn
@@ -1706,11 +1743,13 @@ class CommandDispatcher:
                          private=private, related=related)
 
     @cmdutils.register(instance='command-dispatcher', scope='window')
-    def fullscreen(self, leave=False):
+    def fullscreen(self, leave=False, enter=False):
         """Toggle fullscreen mode.
 
         Args:
             leave: Only leave fullscreen if it was entered by the page.
+            enter: Activate fullscreen and do not toggle if it is already
+                   active.
         """
         if leave:
             tab = self._current_widget()
@@ -1724,7 +1763,10 @@ class CommandDispatcher:
 
         if not window.isFullScreen():
             window.state_before_fullscreen = window.windowState()
-        window.setWindowState(window.windowState() ^ Qt.WindowFullScreen)
+        if enter:
+            window.setWindowState(window.windowState() | Qt.WindowFullScreen)
+        else:
+            window.setWindowState(window.windowState() ^ Qt.WindowFullScreen)
 
         log.misc.debug('state before fullscreen: {}'.format(
             debug.qflags_key(Qt, window.state_before_fullscreen)))
