@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2016-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2016-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -23,20 +23,23 @@ import math
 import functools
 import re
 import html as html_utils
+import typing
+import textwrap
 
 from PyQt5.QtCore import (pyqtSignal, pyqtSlot, Qt, QPoint, QPointF, QUrl,
                           QTimer, QObject)
 from PyQt5.QtNetwork import QAuthenticator
-from PyQt5.QtWidgets import QApplication
+from PyQt5.QtWidgets import QApplication, QWidget
 from PyQt5.QtWebEngineWidgets import QWebEnginePage, QWebEngineScript
 
 from qutebrowser.config import configdata, config
-from qutebrowser.browser import browsertab, mouse, shared, webelem
+from qutebrowser.browser import (browsertab, eventfilter, shared, webelem,
+                                 history, greasemonkey)
 from qutebrowser.browser.webengine import (webview, webengineelem, tabhistory,
                                            interceptor, webenginequtescheme,
                                            cookies, webenginedownloads,
                                            webenginesettings, certificateerror)
-from qutebrowser.misc import miscwidgets
+from qutebrowser.misc import miscwidgets, objects
 from qutebrowser.utils import (usertypes, qtutils, log, javascript, utils,
                                message, objreg, jinja, debug)
 from qutebrowser.qt import sip
@@ -60,8 +63,7 @@ def init():
         _qute_scheme_handler.install(webenginesettings.private_profile)
 
     log.init.debug("Initializing request interceptor...")
-    args = objreg.get('args')
-    req_interceptor = interceptor.RequestInterceptor(args=args, parent=app)
+    req_interceptor = interceptor.RequestInterceptor(parent=app)
     req_interceptor.install(webenginesettings.default_profile)
     if webenginesettings.private_profile:
         req_interceptor.install(webenginesettings.private_profile)
@@ -79,14 +81,13 @@ def init():
         cookies.install_filter(webenginesettings.private_profile)
 
     # Clear visited links on web history clear
-    hist = objreg.get('web-history')
     for p in [webenginesettings.default_profile,
               webenginesettings.private_profile]:
         if not p:
             continue
-        hist.history_cleared.connect(p.clearAllVisitedLinks)
-        hist.url_cleared.connect(lambda url, profile=p:
-                                 profile.clearVisitedLinks([url]))
+        history.web_history.history_cleared.connect(p.clearAllVisitedLinks)
+        history.web_history.url_cleared.connect(
+            lambda url, profile=p: profile.clearVisitedLinks([url]))
 
 
 # Mapping worlds from usertypes.JsWorld to QWebEngineScript world IDs.
@@ -123,7 +124,8 @@ class WebEngineAction(browsertab.AbstractAction):
             # Qt < 5.8
             tb = objreg.get('tabbed-browser', scope='window',
                             window=self._tab.win_id)
-            urlstr = self._tab.url().toString(QUrl.RemoveUserInfo)
+            urlstr = self._tab.url().toString(
+                QUrl.RemoveUserInfo)  # type: ignore
             # The original URL becomes the path of a view-source: URL
             # (without a host), but query/fragment should stay.
             url = QUrl('view-source:' + urlstr)
@@ -167,7 +169,7 @@ class WebEngineSearch(browsertab.AbstractSearch):
 
     def __init__(self, tab, parent=None):
         super().__init__(tab, parent)
-        self._flags = QWebEnginePage.FindFlags(0)
+        self._flags = QWebEnginePage.FindFlags(0)  # type: ignore
         self._pending_searches = 0
 
     def _find(self, text, flags, callback, caller):
@@ -217,7 +219,7 @@ class WebEngineSearch(browsertab.AbstractSearch):
             return
 
         self.text = text
-        self._flags = QWebEnginePage.FindFlags(0)
+        self._flags = QWebEnginePage.FindFlags(0)  # type: ignore
         if self._is_case_sensitive(ignore_case):
             self._flags |= QWebEnginePage.FindCaseSensitively
         if reverse:
@@ -233,7 +235,7 @@ class WebEngineSearch(browsertab.AbstractSearch):
 
     def prev_result(self, *, result_cb=None):
         # The int() here makes sure we get a copy of the flags.
-        flags = QWebEnginePage.FindFlags(int(self._flags))
+        flags = QWebEnginePage.FindFlags(int(self._flags))  # type: ignore
         if flags & QWebEnginePage.FindBackward:
             flags &= ~QWebEnginePage.FindBackward
         else:
@@ -419,7 +421,6 @@ class WebEngineScroller(browsertab.AbstractScroller):
 
     def __init__(self, tab, parent=None):
         super().__init__(tab, parent)
-        self._args = objreg.get('args')
         self._pos_perc = (0, 0)
         self._pos_px = QPoint()
         self._at_bottom = False
@@ -448,7 +449,7 @@ class WebEngineScroller(browsertab.AbstractScroller):
                 perc_x = min(100, round(100 / scrollable_x * pos.x()))
             except ValueError:
                 # https://github.com/qutebrowser/qutebrowser/issues/3219
-                log.misc.debug("Got ValueError!")
+                log.misc.debug("Got ValueError for perc_x!")
                 log.misc.debug("contents_size.width(): {}".format(
                     contents_size.width()))
                 log.misc.debug("self._widget.width(): {}".format(
@@ -461,12 +462,23 @@ class WebEngineScroller(browsertab.AbstractScroller):
         if scrollable_y == 0:
             perc_y = 0
         else:
-            perc_y = min(100, round(100 / scrollable_y * pos.y()))
+            try:
+                perc_y = min(100, round(100 / scrollable_y * pos.y()))
+            except ValueError:
+                # https://github.com/qutebrowser/qutebrowser/issues/3219
+                log.misc.debug("Got ValueError for perc_y!")
+                log.misc.debug("contents_size.height(): {}".format(
+                    contents_size.height()))
+                log.misc.debug("self._widget.height(): {}".format(
+                    self._widget.height()))
+                log.misc.debug("scrollable_y: {}".format(scrollable_y))
+                log.misc.debug("pos.y(): {}".format(pos.y()))
+                raise
 
         self._at_bottom = math.ceil(pos.y()) >= scrollable_y
 
         if (self._pos_perc != (perc_x, perc_y) or
-                'no-scroll-filtering' in self._args.debug_flags):
+                'no-scroll-filtering' in objects.debug_flags):
             self._pos_perc = perc_x, perc_y
             self.perc_changed.emit(*self._pos_perc)
 
@@ -748,7 +760,7 @@ class _WebEnginePermissions(QObject):
     def __init__(self, tab, parent=None):
         super().__init__(parent)
         self._tab = tab
-        self._widget = None
+        self._widget = typing.cast(QWidget, None)
 
         try:
             self._options.update({
@@ -811,25 +823,36 @@ class _WebEnginePermissions(QObject):
     def _on_feature_permission_requested(self, url, feature):
         """Ask the user for approval for geolocation/media/etc.."""
         page = self._widget.page()
+        grant_permission = functools.partial(
+            page.setFeaturePermission, url, feature,
+            QWebEnginePage.PermissionGrantedByUser)
+        deny_permission = functools.partial(
+            page.setFeaturePermission, url, feature,
+            QWebEnginePage.PermissionDeniedByUser)
 
         if feature not in self._options:
             log.webview.error("Unhandled feature permission {}".format(
                 debug.qenum_key(QWebEnginePage, feature)))
-            page.setFeaturePermission(url, feature,
-                                      QWebEnginePage.PermissionDeniedByUser)
+            deny_permission()
             return
 
-        yes_action = functools.partial(
-            page.setFeaturePermission, url, feature,
-            QWebEnginePage.PermissionGrantedByUser)
-        no_action = functools.partial(
-            page.setFeaturePermission, url, feature,
-            QWebEnginePage.PermissionDeniedByUser)
+        if (
+                hasattr(QWebEnginePage, 'DesktopVideoCapture') and
+                feature in [QWebEnginePage.DesktopVideoCapture,
+                            QWebEnginePage.DesktopAudioVideoCapture] and
+                qtutils.version_check('5.13', compiled=False) and
+                not qtutils.version_check('5.13.2', compiled=False)
+        ):
+            # WORKAROUND for https://bugreports.qt.io/browse/QTBUG-78016
+            log.webview.warning("Ignoring desktop sharing request due to "
+                                "crashes in Qt < 5.13.2")
+            deny_permission()
+            return
 
         question = shared.feature_permission(
             url=url.adjusted(QUrl.RemovePath),
             option=self._options[feature], msg=self._messages[feature],
-            yes_action=yes_action, no_action=no_action,
+            yes_action=grant_permission, no_action=deny_permission,
             abort_on=[self._tab.abort_questions])
 
         if question is not None:
@@ -876,8 +899,8 @@ class _WebEngineScripts(QObject):
     def __init__(self, tab, parent=None):
         super().__init__(parent)
         self._tab = tab
-        self._widget = None
-        self._greasemonkey = objreg.get('greasemonkey')
+        self._widget = typing.cast(QWidget, None)
+        self._greasemonkey = greasemonkey.gm_manager
 
     def connect_signals(self):
         """Connect signals to our private slots."""
@@ -963,6 +986,7 @@ class _WebEngineScripts(QObject):
             self._greasemonkey.scripts_reloaded.connect(
                 self._inject_all_greasemonkey_scripts)
             self._inject_all_greasemonkey_scripts()
+            self._inject_site_specific_quirks()
 
     def _init_stylesheet(self):
         """Initialize custom stylesheets.
@@ -1070,6 +1094,36 @@ class _WebEngineScripts(QObject):
                                    .format(new_script.name()))
             page_scripts.insert(new_script)
 
+    def _inject_site_specific_quirks(self):
+        """Add site-specific quirk scripts.
+
+        NOTE: This isn't implemented for Qt 5.7 because of different UserScript
+        semantics there. We only have a quirk for WhatsApp Web right now. It
+        looks like that quirk isn't needed for Qt < 5.13.
+        """
+        if not config.val.content.site_specific_quirks:
+            return
+
+        # WhatsApp Web, based on:
+        # https://github.com/jiahaog/nativefier/issues/719#issuecomment-443809630
+        script = QWebEngineScript()
+        script.setName('quirk-whatsapp')
+        script.setWorldId(QWebEngineScript.ApplicationWorld)
+        script.setInjectionPoint(QWebEngineScript.DocumentReady)
+        script.setSourceCode(textwrap.dedent(r"""
+            // ==UserScript==
+            // @include https://web.whatsapp.com/
+            // ==/UserScript==
+            if (document.body.innerText.replace(/\n/g, ' ').search(
+                     /whatsapp works with.*to use whatsapp.*update/i) !== -1) {
+                navigator.serviceWorker.getRegistration().then(function (r) {
+                    r.unregister();
+                    document.location.reload();
+                });
+            }
+        """))
+        self._widget.page().scripts().insert(script)
+
 
 class WebEngineTabPrivate(browsertab.AbstractTabPrivate):
 
@@ -1129,7 +1183,7 @@ class WebEngineTab(browsertab.AbstractTab):
         self.backend = usertypes.Backend.QtWebEngine
         self._child_event_filter = None
         self._saved_zoom = None
-        self._reload_url = None
+        self._reload_url = None  # type: typing.Optional[QUrl]
         self._scripts.init()
 
     def _set_widget(self, widget):
@@ -1141,9 +1195,9 @@ class WebEngineTab(browsertab.AbstractTab):
     def _install_event_filter(self):
         fp = self._widget.focusProxy()
         if fp is not None:
-            fp.installEventFilter(self._mouse_event_filter)
-        self._child_event_filter = mouse.ChildEventFilter(
-            eventfilter=self._mouse_event_filter, widget=self._widget,
+            fp.installEventFilter(self._tab_event_filter)
+        self._child_event_filter = eventfilter.ChildEventFilter(
+            eventfilter=self._tab_event_filter, widget=self._widget,
             win_id=self.win_id, parent=self)
         self._widget.installEventFilter(self._child_event_filter)
 
@@ -1187,8 +1241,9 @@ class WebEngineTab(browsertab.AbstractTab):
             self._widget.page().toHtml(callback)
 
     def run_js_async(self, code, callback=None, *, world=None):
+        world_id_type = typing.Union[QWebEngineScript.ScriptWorldId, int]
         if world is None:
-            world_id = QWebEngineScript.ApplicationWorld
+            world_id = QWebEngineScript.ApplicationWorld  # type: world_id_type
         elif isinstance(world, int):
             world_id = world
             if not 0 <= world_id <= qtutils.MAX_WORLD_ID:
@@ -1255,7 +1310,9 @@ class WebEngineTab(browsertab.AbstractTab):
         title = self.title()
         title_url = QUrl(url)
         title_url.setScheme('')
-        if title == title_url.toDisplayString(QUrl.RemoveScheme).strip('/'):
+        title_url_str = title_url.toDisplayString(
+            QUrl.RemoveScheme)  # type: ignore
+        if title == title_url_str.strip('/'):
             title = ""
 
         # Don't add history entry if the URL is invalid anyways
@@ -1281,7 +1338,7 @@ class WebEngineTab(browsertab.AbstractTab):
             authenticator.setPassword(answer.password)
         else:
             try:
-                sip.assign(authenticator, QAuthenticator())
+                sip.assign(authenticator, QAuthenticator())  # type: ignore
             except AttributeError:
                 self._show_error_page(url, "Proxy authentication required")
 
@@ -1302,7 +1359,7 @@ class WebEngineTab(browsertab.AbstractTab):
         if not netrc_success and answer is None:
             log.network.debug("Aborting auth")
             try:
-                sip.assign(authenticator, QAuthenticator())
+                sip.assign(authenticator, QAuthenticator())  # type: ignore
             except AttributeError:
                 # WORKAROUND for
                 # https://www.riverbankcomputing.com/pipermail/pyqt/2016-December/038400.html
@@ -1340,16 +1397,23 @@ class WebEngineTab(browsertab.AbstractTab):
         }
         self.renderer_process_terminated.emit(status_map[status], exitcode)
 
-    def _error_page_workaround(self, html):
+    def _error_page_workaround(self, js_enabled, html):
         """Check if we're displaying a Chromium error page.
 
-        This gets only called if we got loadFinished(False) without JavaScript,
-        so we can display at least some error page.
+        This gets called if we got a loadFinished(False), so we can display at
+        least some error page in situations where Chromium's can't be
+        displayed.
 
         WORKAROUND for https://bugreports.qt.io/browse/QTBUG-66643
+        WORKAROUND for https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=882805
+
         Needs to check the page content as a WORKAROUND for
         https://bugreports.qt.io/browse/QTBUG-66661
         """
+        missing_jst = 'jstProcess(' in html and 'jstProcess=' not in html
+        if js_enabled and not missing_jst:
+            return
+
         match = re.search(r'"errorCode":"([^"]*)"', html)
         if match is None:
             return
@@ -1379,9 +1443,10 @@ class WebEngineTab(browsertab.AbstractTab):
         else:
             self._update_load_status(ok)
 
-        js_enabled = self.settings.test_attribute('content.javascript.enabled')
-        if not ok and not js_enabled:
-            self.dump_async(self._error_page_workaround)
+        if not ok:
+            self.dump_async(functools.partial(
+                self._error_page_workaround,
+                self.settings.test_attribute('content.javascript.enabled')))
 
         if ok and self._reload_url is not None:
             # WORKAROUND for https://bugreports.qt.io/browse/QTBUG-66656
@@ -1411,14 +1476,32 @@ class WebEngineTab(browsertab.AbstractTab):
             error.ignore, url, self.url(requested=True)))
 
         # WORKAROUND for https://bugreports.qt.io/browse/QTBUG-56207
+        show_cert_error = (
+            not qtutils.version_check('5.9') and
+            not error.ignore
+        )
+        # WORKAROUND for https://codereview.qt-project.org/c/qt/qtwebengine/+/270556
+        show_non_overr_cert_error = (
+            not error.is_overridable() and (
+                # Affected Qt versions:
+                # 5.13 before 5.13.2
+                # 5.12 before 5.12.6
+                # < 5.12
+                (qtutils.version_check('5.13') and
+                 not qtutils.version_check('5.13.2')) or
+                (qtutils.version_check('5.12') and
+                 not qtutils.version_check('5.12.6')) or
+                not qtutils.version_check('5.12')
+            )
+        )
+
         # We can't really know when to show an error page, as the error might
         # have happened when loading some resource.
         # However, self.url() is not available yet and the requested URL
         # might not match the URL we get from the error - so we just apply a
         # heuristic here.
-        if (not qtutils.version_check('5.9') and
-                not error.ignore and
-                url.matches(self.url(requested=True), QUrl.RemoveScheme)):
+        if ((show_cert_error or show_non_overr_cert_error) and
+                url.matches(self.data.last_navigation.url, QUrl.RemoveScheme)):
             self._show_error_page(url, str(error))
 
     @pyqtSlot(QUrl)
@@ -1439,6 +1522,21 @@ class WebEngineTab(browsertab.AbstractTab):
         except browsertab.WebTabError as e:
             message.error(str(e))
 
+    @pyqtSlot(QUrl)
+    def _on_url_changed(self, url: QUrl) -> None:
+        """Update settings for the current URL.
+
+        Normally this is done below in _on_navigation_request, but we also need
+        to do it here as WORKAROUND for
+        https://bugreports.qt.io/browse/QTBUG-77137
+
+        Since update_for_url() is idempotent, it doesn't matter much if we end
+        up doing it twice.
+        """
+        super()._on_url_changed(url)
+        if url.isValid() and qtutils.version_check('5.13'):
+            self.settings.update_for_url(url)
+
     @pyqtSlot(usertypes.NavigationRequest)
     def _on_navigation_request(self, navigation):
         super()._on_navigation_request(navigation)
@@ -1460,7 +1558,7 @@ class WebEngineTab(browsertab.AbstractTab):
         assert settings_needing_reload.issubset(configdata.DATA)
 
         changed = self.settings.update_for_url(navigation.url)
-        reload_needed = changed & settings_needing_reload
+        reload_needed = bool(changed & settings_needing_reload)
 
         # On Qt < 5.11, we don't don't need a reload when type == link_clicked.
         # On Qt 5.11.0, we always need a reload.
@@ -1527,7 +1625,7 @@ class WebEngineTab(browsertab.AbstractTab):
 
         try:
             # pylint: disable=unused-import
-            from PyQt5.QtWebEngineWidgets import (
+            from PyQt5.QtWebEngineWidgets import (  # type: ignore
                 QWebEngineClientCertificateSelection)
         except ImportError:
             pass
@@ -1546,8 +1644,8 @@ class WebEngineTab(browsertab.AbstractTab):
         page.loadFinished.connect(self._on_load_finished)
 
         self.before_load_started.connect(self._on_before_load_started)
-        self.shutting_down.connect(self.abort_questions)
-        self.load_started.connect(self.abort_questions)
+        self.shutting_down.connect(self.abort_questions)  # type: ignore
+        self.load_started.connect(self.abort_questions)  # type: ignore
 
         # pylint: disable=protected-access
         self.audio._connect_signals()
