@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2015-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2015-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 # Copyright 2015-2018 Daniel Schadt
 #
 # This file is part of qutebrowser.
@@ -33,6 +33,7 @@ import email.encoders
 import email.mime.multipart
 import email.message
 import quopri
+import typing
 
 import attr
 from PyQt5.QtCore import QUrl
@@ -188,7 +189,7 @@ class MHTMLWriter:
         self.root_content = root_content
         self.content_location = content_location
         self.content_type = content_type
-        self._files = {}
+        self._files = {}  # type: typing.MutableMapping[QUrl, _File]
 
     def add_file(self, location, content, content_type=None,
                  transfer_encoding=E_QUOPRI):
@@ -243,6 +244,10 @@ class MHTMLWriter:
         return msg
 
 
+_PendingDownloadType = typing.Set[
+    typing.Tuple[QUrl, downloads.AbstractDownloadItem]]
+
+
 class _Downloader:
 
     """A class to download whole websites.
@@ -263,7 +268,7 @@ class _Downloader:
         self.target = target
         self.writer = None
         self.loaded_urls = {tab.url()}
-        self.pending_downloads = set()
+        self.pending_downloads = set()  # type: _PendingDownloadType
         self._finished_file = False
         self._used = False
 
@@ -343,6 +348,8 @@ class _Downloader:
         Args:
             url: The file to download as QUrl.
         """
+        assert self.writer is not None
+
         if url.scheme() not in ['http', 'https']:
             return
         # Prevent loading an asset twice
@@ -382,6 +389,8 @@ class _Downloader:
             url: The original url of the asset as QUrl.
             item: The DownloadItem given by the DownloadManager
         """
+        assert self.writer is not None
+
         self.pending_downloads.remove((url, item))
         mime = item.raw_headers.get(b'Content-Type', b'')
 
@@ -432,6 +441,7 @@ class _Downloader:
             url: The original url of the asset as QUrl.
             item: The DownloadItem given by the DownloadManager.
         """
+        assert self.writer is not None
         try:
             self.pending_downloads.remove((url, item))
         except KeyError:
@@ -462,6 +472,8 @@ class _Downloader:
 
     def _finish_file(self):
         """Save the file to the filename given in __init__."""
+        assert self.writer is not None
+
         if self._finished_file:
             log.downloads.debug("finish_file called twice, ignored!")
             return
