@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -24,7 +24,7 @@ from PyQt5.QtWidgets import QStyleFactory
 from PyQt5.QtWebKit import QWebSettings
 from PyQt5.QtWebKitWidgets import QWebView, QWebPage
 
-from qutebrowser.config import config
+from qutebrowser.config import config, stylesheet
 from qutebrowser.keyinput import modeman
 from qutebrowser.utils import log, usertypes, utils, objreg, debug
 from qutebrowser.browser.webkit import webpage
@@ -51,7 +51,9 @@ class WebView(QWebView):
 
     STYLESHEET = """
         WebView {
+            {% if conf.colors.webpage.bg %}
             background-color: {{ qcolor_to_qsscolor(conf.colors.webpage.bg) }};
+            {% endif %}
         }
     """
 
@@ -82,10 +84,11 @@ class WebView(QWebView):
 
         self.setPage(page)
 
-        config.set_register_stylesheet(self)
+        stylesheet.set_register(self)
 
     def __repr__(self):
-        url = utils.elide(self.url().toDisplayString(QUrl.EncodeUnicode), 100)
+        urlstr = self.url().toDisplayString(QUrl.EncodeUnicode)  # type: ignore
+        url = utils.elide(urlstr, 100)
         return utils.get_repr(self, tab_id=self._tab_id, url=url)
 
     def __del__(self):
@@ -94,7 +97,7 @@ class WebView(QWebView):
         # Copied from:
         # https://code.google.com/p/webscraping/source/browse/webkit.py#325
         try:
-            self.setPage(None)
+            self.setPage(None)  # type: ignore
         except RuntimeError:
             # It seems sometimes Qt has already deleted the QWebView and we
             # get: RuntimeError: wrapped C/C++ object of type WebView has been
@@ -177,8 +180,9 @@ class WebView(QWebView):
         This is not needed for QtWebEngine, so it's in here.
         """
         menu = self.page().createStandardContextMenu()
-        self.shutting_down.connect(menu.close)
-        modeman.instance(self.win_id).entered.connect(menu.close)
+        self.shutting_down.connect(menu.close)  # type: ignore
+        mm = modeman.instance(self.win_id)
+        mm.entered.connect(menu.close)  # type: ignore
         menu.exec_(e.globalPos())
 
     def showEvent(self, e):
