@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -32,9 +32,7 @@ from qutebrowser.config import configtypes, configexc, configfiles, configdata
 from qutebrowser.misc import editor
 from qutebrowser.keyinput import keyutils
 
-MYPY = False
-if MYPY:
-    # pylint: disable=unused-import,useless-suppression
+if typing.TYPE_CHECKING:
     from qutebrowser.config.config import Config, KeyConfig
 
 
@@ -397,6 +395,8 @@ class ConfigCommands:
             filename = standarddir.config_py()
         else:
             filename = os.path.expanduser(filename)
+            if not os.path.isabs(filename):
+                filename = os.path.join(standarddir.config(), filename)
 
         if clear:
             self.config_clear()
