@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -56,8 +56,8 @@ import typing
 import attr
 import yaml
 from PyQt5.QtCore import QUrl, Qt
-from PyQt5.QtGui import QColor, QFont
-from PyQt5.QtWidgets import QTabWidget, QTabBar
+from PyQt5.QtGui import QColor, QFont, QFontDatabase
+from PyQt5.QtWidgets import QTabWidget, QTabBar, QApplication
 from PyQt5.QtNetwork import QNetworkProxy
 
 from qutebrowser.misc import objects, debugcachestats
@@ -80,8 +80,8 @@ BOOLEAN_STATES = {'1': True, 'yes': True, 'true': True, 'on': True,
 
 
 _Completions = typing.Optional[typing.Iterable[typing.Tuple[str, str]]]
-_StrUnset = typing.Union[str, configutils.Unset]
-_StrUnsetNone = typing.Union[None, str, configutils.Unset]
+_StrUnset = typing.Union[str, usertypes.Unset]
+_StrUnsetNone = typing.Union[None, str, usertypes.Unset]
 
 
 class ValidValues:
@@ -168,7 +168,7 @@ class BaseType:
             value: The value to check.
             pytype: A Python type to check the value against.
         """
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return
 
         if (value is None or (pytype == list and value == []) or
@@ -342,7 +342,7 @@ class MappingType(BaseType):
 
     def to_py(self, value: typing.Any) -> typing.Any:
         self._basic_py_validation(value, str)
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return None
@@ -364,12 +364,13 @@ class String(BaseType):
         minlen: Minimum length (inclusive).
         maxlen: Maximum length (inclusive).
         forbidden: Forbidden chars in the string.
+        regex: A regex used to validate the string.
         completions: completions to be used, or None
     """
 
     def __init__(self, *, minlen: int = None, maxlen: int = None,
-                 forbidden: str = None, encoding: str = None,
-                 none_ok: bool = False,
+                 forbidden: str = None, regex: str = None,
+                 encoding: str = None, none_ok: bool = False,
                  completions: _Completions = None,
                  valid_values: ValidValues = None) -> None:
         super().__init__(none_ok)
@@ -387,6 +388,7 @@ class String(BaseType):
         self.forbidden = forbidden
         self._completions = completions
         self.encoding = encoding
+        self.regex = regex
 
     def _validate_encoding(self, value: str) -> None:
         """Check if the given value fits into the configured encoding.
@@ -408,7 +410,7 @@ class String(BaseType):
 
     def to_py(self, value: _StrUnset) -> _StrUnsetNone:
         self._basic_py_validation(value, str)
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return None
@@ -426,6 +428,9 @@ class String(BaseType):
         if self.maxlen is not None and len(value) > self.maxlen:
             raise configexc.ValidationError(value, "must be at most {} chars "
                                             "long!".format(self.maxlen))
+        if self.regex is not None and not re.fullmatch(self.regex, value):
+            raise configexc.ValidationError(value, "does not match {}"
+                                            .format(self.regex))
 
         return value
 
@@ -440,7 +445,7 @@ class String(BaseType):
                               valid_values=self.valid_values,
                               minlen=self.minlen,
                               maxlen=self.maxlen, forbidden=self.forbidden,
-                              completions=self._completions,
+                              regex=self.regex, completions=self._completions,
                               encoding=self.encoding)
 
 
@@ -450,7 +455,7 @@ class UniqueCharString(String):
 
     def to_py(self, value: _StrUnset) -> _StrUnsetNone:
         py_value = super().to_py(value)
-        if isinstance(py_value, configutils.Unset):
+        if isinstance(py_value, usertypes.Unset):
             return py_value
         elif not py_value:
             return None
@@ -510,10 +515,10 @@ class List(BaseType):
 
     def to_py(
             self,
-            value: typing.Union[typing.List, configutils.Unset]
-    ) -> typing.Union[typing.List, configutils.Unset]:
+            value: typing.Union[typing.List, usertypes.Unset]
+    ) -> typing.Union[typing.List, usertypes.Unset]:
         self._basic_py_validation(value, list)
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return []
@@ -601,7 +606,7 @@ class ListOrValue(BaseType):
         return value
 
     def to_py(self, value: typing.Any) -> typing.Any:
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
 
         try:
@@ -652,10 +657,10 @@ class FlagList(List):
 
     def to_py(
             self,
-            value: typing.Union[configutils.Unset, typing.List],
-    ) -> typing.Union[configutils.Unset, typing.List]:
+            value: typing.Union[usertypes.Unset, typing.List],
+    ) -> typing.Union[usertypes.Unset, typing.List]:
         vals = super().to_py(value)
-        if not isinstance(vals, configutils.Unset):
+        if not isinstance(vals, usertypes.Unset):
             self._check_duplicates(vals)
         return vals
 
@@ -740,7 +745,7 @@ class BoolAsk(Bool):
                  value: str) -> typing.Union[bool, str, None]:
         # basic validation unneeded if it's == 'ask' and done by Bool if we
         # call super().from_str
-        if isinstance(value, str) and value.lower() == 'ask':
+        if value.lower() == 'ask':
             return 'ask'
         return super().from_str(value)
 
@@ -765,10 +770,12 @@ class _Numeric(BaseType):  # pylint: disable=abstract-method
 
     def __init__(self, minval: int = None,
                  maxval: int = None,
+                 zero_ok: bool = True,
                  none_ok: bool = False) -> None:
         super().__init__(none_ok)
         self.minval = self._parse_bound(minval)
         self.maxval = self._parse_bound(maxval)
+        self.zero_ok = zero_ok
         if self.maxval is not None and self.minval is not None:
             if self.maxval < self.minval:
                 raise ValueError("minval ({}) needs to be <= maxval ({})!"
@@ -798,6 +805,8 @@ class _Numeric(BaseType):  # pylint: disable=abstract-method
         elif self.maxval is not None and value > self.maxval:
             raise configexc.ValidationError(
                 value, "must be {}{} or smaller!".format(self.maxval, suffix))
+        elif not self.zero_ok and value == 0:
+            raise configexc.ValidationError(value, "must not be 0!")
 
     def to_str(self, value: typing.Union[None, int, float]) -> str:
         if value is None:
@@ -862,10 +871,10 @@ class Perc(_Numeric):
 
     def to_py(
             self,
-            value: typing.Union[None, float, int, str, configutils.Unset]
-    ) -> typing.Union[None, float, int, configutils.Unset]:
+            value: typing.Union[None, float, int, str, usertypes.Unset]
+    ) -> typing.Union[None, float, int, usertypes.Unset]:
         self._basic_py_validation(value, (float, int, str))
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return None
@@ -1058,10 +1067,10 @@ class QtColor(BaseType):
         except ValueError:
             raise configexc.ValidationError(val, "must be a valid color value")
 
-    def to_py(self, value: _StrUnset) -> typing.Union[configutils.Unset,
+    def to_py(self, value: _StrUnset) -> typing.Union[usertypes.Unset,
                                                       None, QColor]:
         self._basic_py_validation(value, str)
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return None
@@ -1082,7 +1091,7 @@ class QtColor(BaseType):
             if not conv:
                 raise configexc.ValidationError(
                     value,
-                    '{} not in {}'.format(kind, list(sorted(converters))))
+                    '{} not in {}'.format(kind, sorted(converters)))
 
             if len(kind) != len(vals):
                 raise configexc.ValidationError(
@@ -1119,7 +1128,7 @@ class QssColor(BaseType):
 
     def to_py(self, value: _StrUnset) -> _StrUnsetNone:
         self._basic_py_validation(value, str)
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return None
@@ -1147,7 +1156,8 @@ class Font(BaseType):
     """
 
     # Gets set when the config is initialized.
-    monospace_fonts = None
+    default_family = None  # type: str
+    default_size = None  # type: str
     font_regex = re.compile(r"""
         (
             (
@@ -1159,14 +1169,69 @@ class Font(BaseType):
                     (?P<namedweight>normal|bold)
                 ) |
                 # size (<float>pt | <int>px)
-                (?P<size>[0-9]+((\.[0-9]+)?[pP][tT]|[pP][xX]))
+                (?P<size>[0-9]+((\.[0-9]+)?[pP][tT]|[pP][xX])|default_size)
             )\           # size/weight/style are space-separated
         )*               # 0-inf size/weight/style tags
         (?P<family>.+)  # mandatory font family""", re.VERBOSE)
 
+    @classmethod
+    def set_defaults(cls, default_family: typing.List[str],
+                     default_size: str) -> None:
+        """Make sure default_family/default_size are available.
+
+        If the given family value (fonts.default_family in the config) is
+        unset, a system-specific default monospace font is used.
+
+        Note that (at least) three ways of getting the default monospace font
+        exist:
+
+        1) f = QFont()
+           f.setStyleHint(QFont.Monospace)
+           print(f.defaultFamily())
+
+        2) f = QFont()
+           f.setStyleHint(QFont.TypeWriter)
+           print(f.defaultFamily())
+
+        3) f = QFontDatabase.systemFont(QFontDatabase.FixedFont)
+           print(f.family())
+
+        They yield different results depending on the OS:
+
+                   QFont.Monospace  | QFont.TypeWriter    | QFontDatabase
+                   ------------------------------------------------------
+        Windows:   Courier New      | Courier New         | Courier New
+        Linux:     DejaVu Sans Mono | DejaVu Sans Mono    | monospace
+        macOS:     Menlo            | American Typewriter | Monaco
+
+        Test script: https://p.cmpl.cc/d4dfe573
+
+        On Linux, it seems like both actually resolve to the same font.
+
+        On macOS, "American Typewriter" looks like it indeed tries to imitate a
+        typewriter, so it's not really a suitable UI font.
+
+        Looking at those Wikipedia articles:
+
+        https://en.wikipedia.org/wiki/Monaco_(typeface)
+        https://en.wikipedia.org/wiki/Menlo_(typeface)
+
+        the "right" choice isn't really obvious. Thus, let's go for the
+        QFontDatabase approach here, since it's by far the simplest one.
+        """
+        if default_family:
+            families = configutils.FontFamilies(default_family)
+        else:
+            assert QApplication.instance() is not None
+            font = QFontDatabase.systemFont(QFontDatabase.FixedFont)
+            families = configutils.FontFamilies([font.family()])
+
+        cls.default_family = families.to_str(quote=True)
+        cls.default_size = default_size
+
     def to_py(self, value: _StrUnset) -> _StrUnsetNone:
         self._basic_py_validation(value, str)
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return None
@@ -1176,8 +1241,13 @@ class Font(BaseType):
             # as family.
             raise configexc.ValidationError(value, "must be a valid font")
 
-        if value.endswith(' monospace') and self.monospace_fonts is not None:
-            return value.replace('monospace', self.monospace_fonts)
+        if (value.endswith(' default_family') and
+                self.default_family is not None):
+            value = value.replace('default_family', self.default_family)
+
+        if 'default_size ' in value and self.default_size is not None:
+            value = value.replace('default_size', self.default_size)
+
         return value
 
 
@@ -1187,7 +1257,7 @@ class FontFamily(Font):
 
     def to_py(self, value: _StrUnset) -> _StrUnsetNone:
         self._basic_py_validation(value, str)
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return None
@@ -1211,23 +1281,20 @@ class QtFont(Font):
 
     __doc__ = Font.__doc__  # for src2asciidoc.py
 
-    def to_py(self, value: _StrUnset) -> typing.Union[configutils.Unset,
-                                                      None, QFont]:
+    def _parse_families(self, family_str: str) -> configutils.FontFamilies:
+        if family_str == 'default_family' and self.default_family is not None:
+            family_str = self.default_family
+
+        return configutils.FontFamilies.from_str(family_str)
+
+    def to_py(self, value: _StrUnset) -> typing.Union[  # noqa: C901
+            usertypes.Unset, None, QFont]:
         self._basic_py_validation(value, str)
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return None
 
-        style_map = {
-            'normal': QFont.StyleNormal,
-            'italic': QFont.StyleItalic,
-            'oblique': QFont.StyleOblique,
-        }
-        weight_map = {
-            'normal': QFont.Normal,
-            'bold': QFont.Bold,
-        }
         font = QFont()
         font.setStyle(QFont.StyleNormal)
         font.setWeight(QFont.Normal)
@@ -1242,16 +1309,34 @@ class QtFont(Font):
         weight = match.group('weight')
         namedweight = match.group('namedweight')
         size = match.group('size')
-        family = match.group('family')
+        family_str = match.group('family')
+
+        style_map = {
+            'normal': QFont.StyleNormal,
+            'italic': QFont.StyleItalic,
+            'oblique': QFont.StyleOblique,
+        }
         if style:
             font.setStyle(style_map[style])
+
+        weight_map = {
+            'normal': QFont.Normal,
+            'bold': QFont.Bold,
+        }
         if namedweight:
             font.setWeight(weight_map[namedweight])
-        if weight:
+        elif weight:
             # based on qcssparser.cpp:setFontWeightFromValue
             font.setWeight(min(int(weight) // 8, 99))
+
         if size:
-            if size.lower().endswith('pt'):
+            if size == 'default_size':
+                size = self.default_size
+
+            if size is None:
+                # initial validation before default_size is set up.
+                pass
+            elif size.lower().endswith('pt'):
                 font.setPointSizeF(float(size[:-2]))
             elif size.lower().endswith('px'):
                 font.setPixelSize(int(size[:-2]))
@@ -1261,14 +1346,13 @@ class QtFont(Font):
                 raise ValueError("Unexpected size unit in {!r}!".format(
                     size))  # pragma: no cover
 
-        if family == 'monospace' and self.monospace_fonts is not None:
-            family = self.monospace_fonts
-        # The Qt CSS parser handles " and ' before passing the string to
-        # QFont.setFamily. We could do proper CSS-like parsing here, but since
-        # hopefully nobody will ever have a font with quotes in the family (if
-        # that's even possible), we take a much more naive approach.
-        family = family.replace('"', '').replace("'", '')
-        font.setFamily(family)
+        families = self._parse_families(family_str)
+        if hasattr(font, 'setFamilies'):
+            # Added in Qt 5.13
+            font.setFamily(families.family)  # type: ignore
+            font.setFamilies(list(families))
+        else:  # pragma: no cover
+            font.setFamily(families.to_str(quote=False))
 
         return font
 
@@ -1329,11 +1413,11 @@ class Regex(BaseType):
 
     def to_py(
             self,
-            value: typing.Union[str, typing.Pattern[str]]
-    ) -> typing.Union[configutils.Unset, None, typing.Pattern[str]]:
+            value: typing.Union[str, typing.Pattern[str], usertypes.Unset]
+    ) -> typing.Union[usertypes.Unset, None, typing.Pattern[str]]:
         """Get a compiled regex from either a string or a regex object."""
         self._basic_py_validation(value, (str, self._regex_type))
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return None
@@ -1421,10 +1505,10 @@ class Dict(BaseType):
 
     def to_py(
             self,
-            value: typing.Union[typing.Dict, configutils.Unset, None]
-    ) -> typing.Union[typing.Dict, configutils.Unset]:
+            value: typing.Union[typing.Dict, usertypes.Unset, None]
+    ) -> typing.Union[typing.Dict, usertypes.Unset]:
         self._basic_py_validation(value, dict)
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return self._fill_fixed_keys({})
@@ -1473,7 +1557,7 @@ class File(BaseType):
 
     def to_py(self, value: _StrUnset) -> _StrUnsetNone:
         self._basic_py_validation(value, str)
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return None
@@ -1505,7 +1589,7 @@ class Directory(BaseType):
 
     def to_py(self, value: _StrUnset) -> _StrUnsetNone:
         self._basic_py_validation(value, str)
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return None
@@ -1526,16 +1610,23 @@ class Directory(BaseType):
 
 class FormatString(BaseType):
 
-    """A string with placeholders."""
+    """A string with placeholders.
+
+    Attributes:
+        fields: Which replacements are allowed in the format string.
+        completions: completions to be used, or None
+    """
 
     def __init__(self, fields: typing.Iterable[str],
-                 none_ok: bool = False) -> None:
+                 none_ok: bool = False,
+                 completions: _Completions = None) -> None:
         super().__init__(none_ok)
         self.fields = fields
+        self._completions = completions
 
     def to_py(self, value: _StrUnset) -> _StrUnsetNone:
         self._basic_py_validation(value, str)
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return None
@@ -1550,6 +1641,12 @@ class FormatString(BaseType):
 
         return value
 
+    def complete(self) -> _Completions:
+        if self._completions is not None:
+            return self._completions
+        else:
+            return super().complete()
+
     def __repr__(self) -> str:
         return utils.get_repr(self, none_ok=self.none_ok, fields=self.fields)
 
@@ -1573,10 +1670,10 @@ class ShellCommand(List):
 
     def to_py(
             self,
-            value: typing.Union[typing.List, configutils.Unset],
-    ) -> typing.Union[typing.List, configutils.Unset]:
+            value: typing.Union[typing.List, usertypes.Unset],
+    ) -> typing.Union[typing.List, usertypes.Unset]:
         py_value = super().to_py(value)
-        if isinstance(py_value, configutils.Unset):
+        if isinstance(py_value, usertypes.Unset):
             return py_value
         elif not py_value:
             return []
@@ -1607,9 +1704,9 @@ class Proxy(BaseType):
     def to_py(
             self,
             value: _StrUnset
-    ) -> typing.Union[configutils.Unset, None, QNetworkProxy, _SystemProxy]:
+    ) -> typing.Union[usertypes.Unset, None, QNetworkProxy, _SystemProxy]:
         self._basic_py_validation(value, str)
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return None
@@ -1649,7 +1746,7 @@ class SearchEngineUrl(BaseType):
 
     def to_py(self, value: _StrUnset) -> _StrUnsetNone:
         self._basic_py_validation(value, str)
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return None
@@ -1679,7 +1776,7 @@ class FuzzyUrl(BaseType):
 
     def to_py(self, value: _StrUnset) -> _StrUnsetNone:
         self._basic_py_validation(value, str)
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return None
@@ -1715,10 +1812,10 @@ class Padding(Dict):
 
     def to_py(  # type: ignore
             self,
-            value: typing.Union[configutils.Unset, typing.Dict, None],
-    ) -> typing.Union[configutils.Unset, PaddingValues]:
+            value: typing.Union[usertypes.Unset, typing.Dict, None],
+    ) -> typing.Union[usertypes.Unset, PaddingValues]:
         d = super().to_py(value)
-        if isinstance(d, configutils.Unset):
+        if isinstance(d, usertypes.Unset):
             return d
 
         return PaddingValues(**d)
@@ -1730,7 +1827,7 @@ class Encoding(BaseType):
 
     def to_py(self, value: _StrUnset) -> _StrUnsetNone:
         self._basic_py_validation(value, str)
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return None
@@ -1790,9 +1887,9 @@ class Url(BaseType):
     def to_py(
             self,
             value: _StrUnset
-    ) -> typing.Union[configutils.Unset, None, QUrl]:
+    ) -> typing.Union[usertypes.Unset, None, QUrl]:
         self._basic_py_validation(value, str)
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return None
@@ -1810,7 +1907,7 @@ class SessionName(BaseType):
 
     def to_py(self, value: _StrUnset) -> _StrUnsetNone:
         self._basic_py_validation(value, str)
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return None
@@ -1860,10 +1957,10 @@ class ConfirmQuit(FlagList):
 
     def to_py(
             self,
-            value: typing.Union[configutils.Unset, typing.List],
-    ) -> typing.Union[typing.List, configutils.Unset]:
+            value: typing.Union[usertypes.Unset, typing.List],
+    ) -> typing.Union[typing.List, usertypes.Unset]:
         values = super().to_py(value)
-        if isinstance(values, configutils.Unset):
+        if isinstance(values, usertypes.Unset):
             return values
         elif not values:
             return []
@@ -1904,9 +2001,9 @@ class Key(BaseType):
     def to_py(
             self,
             value: _StrUnset
-    ) -> typing.Union[configutils.Unset, None, keyutils.KeySequence]:
+    ) -> typing.Union[usertypes.Unset, None, keyutils.KeySequence]:
         self._basic_py_validation(value, str)
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return None
@@ -1928,9 +2025,9 @@ class UrlPattern(BaseType):
     def to_py(
             self,
             value: _StrUnset
-    ) -> typing.Union[configutils.Unset, None, urlmatch.UrlPattern]:
+    ) -> typing.Union[usertypes.Unset, None, urlmatch.UrlPattern]:
         self._basic_py_validation(value, str)
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return None
