@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2016-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2016-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -22,6 +22,8 @@
 import os.path
 import html
 import collections
+import functools
+import typing
 
 import attr
 from PyQt5.QtCore import (pyqtSlot, pyqtSignal, Qt, QTimer, QDir, QModelIndex,
@@ -31,14 +33,14 @@ from PyQt5.QtWidgets import (QWidget, QGridLayout, QVBoxLayout, QLineEdit,
                              QSpacerItem)
 
 from qutebrowser.browser import downloads
-from qutebrowser.config import config
+from qutebrowser.config import config, configtypes, configexc, stylesheet
 from qutebrowser.utils import usertypes, log, utils, qtutils, objreg, message
 from qutebrowser.keyinput import modeman
 from qutebrowser.api import cmdutils
-from qutebrowser.qt import sip
+from qutebrowser.utils import urlmatch
 
 
-prompt_queue = None
+prompt_queue = typing.cast('PromptQueue', None)
 
 
 @attr.s
@@ -55,7 +57,7 @@ class Error(Exception):
     """Base class for errors in this module."""
 
 
-class UnsupportedOperationError(Exception):
+class UnsupportedOperationError(Error):
 
     """Raised when the prompt class doesn't support the requested operation."""
 
@@ -100,8 +102,9 @@ class PromptQueue(QObject):
         super().__init__(parent)
         self._question = None
         self._shutting_down = False
-        self._loops = []
-        self._queue = collections.deque()
+        self._loops = []  # type: typing.MutableSequence[qtutils.EventLoop]
+        self._queue = collections.deque(
+        )  # type: typing.Deque[usertypes.Question]
         message.global_bridge.mode_left.connect(self._on_mode_left)
 
     def __repr__(self):
@@ -117,10 +120,11 @@ class PromptQueue(QObject):
         log.prompt.debug("Popping from queue {}".format(self._queue))
         if self._queue:
             question = self._queue.popleft()
-            if not sip.isdeleted(question):
-                # the question could already be deleted, e.g. by a cancelled
+            if not question.is_aborted:
+                # the question could already be aborted, e.g. by a cancelled
                 # download. See
-                # https://github.com/qutebrowser/qutebrowser/issues/415
+                # https://github.com/qutebrowser/qutebrowser/issues/415 and
+                # https://github.com/qutebrowser/qutebrowser/issues/1249
                 self.ask_question(question, blocking=False)
 
     def shutdown(self):
@@ -188,7 +192,8 @@ class PromptQueue(QObject):
         if blocking:
             loop = qtutils.EventLoop()
             self._loops.append(loop)
-            loop.destroyed.connect(lambda: self._loops.remove(loop))
+            loop.destroyed.connect(  # type: ignore
+                lambda: self._loops.remove(loop))
             question.completed.connect(loop.quit)
             question.completed.connect(loop.deleteLater)
             log.prompt.debug("Starting loop.exec_() for {}".format(question))
@@ -283,11 +288,11 @@ class PromptContainer(QWidget):
         self._layout = QVBoxLayout(self)
         self._layout.setContentsMargins(10, 10, 10, 10)
         self._win_id = win_id
-        self._prompt = None
+        self._prompt = None  # type: typing.Optional[_BasePrompt]
 
         self.setObjectName('PromptContainer')
         self.setAttribute(Qt.WA_StyledBackground, True)
-        config.set_register_stylesheet(self)
+        stylesheet.set_register(self)
 
         message.global_bridge.prompt_done.connect(self._on_prompt_done)
         prompt_queue.show_prompts.connect(self._on_show_prompts)
@@ -324,16 +329,15 @@ class PromptContainer(QWidget):
             usertypes.PromptMode.alert: AlertPrompt,
         }
         klass = classes[question.mode]
-        prompt = klass(question)
+        prompt = typing.cast(_BasePrompt, klass(question))
 
         log.prompt.debug("Displaying prompt {}".format(prompt))
         self._prompt = prompt
 
+        # If this question was interrupted, we already connected the signal
         if not question.interrupted:
-            # If this question was interrupted, we already connected the signal
             question.aborted.connect(
-                lambda: modeman.leave(self._win_id, prompt.KEY_MODE, 'aborted',
-                                      maybe=True))
+                functools.partial(self._on_aborted, prompt.KEY_MODE))
         modeman.enter(self._win_id, prompt.KEY_MODE, 'question asked')
 
         self.setSizePolicy(prompt.sizePolicy())
@@ -343,6 +347,15 @@ class PromptContainer(QWidget):
         prompt.setFocus()
         self.update_geometry.emit()
 
+    @pyqtSlot()
+    def _on_aborted(self, key_mode):
+        """Leave KEY_MODE whenever a prompt is aborted."""
+        try:
+            modeman.leave(self._win_id, key_mode, 'aborted', maybe=True)
+        except objreg.RegistryUnavailableError:
+            # window was deleted: ignore
+            pass
+
     @pyqtSlot(usertypes.KeyMode)
     def _on_prompt_done(self, key_mode):
         """Leave the prompt mode in this window if a question was answered."""
@@ -368,7 +381,7 @@ class PromptContainer(QWidget):
     @cmdutils.register(instance='prompt-container', scope='window',
                        modes=[usertypes.KeyMode.prompt,
                               usertypes.KeyMode.yesno])
-    def prompt_accept(self, value=None):
+    def prompt_accept(self, value=None, *, save=False):
         """Accept the current prompt.
 
         //
@@ -379,12 +392,16 @@ class PromptContainer(QWidget):
         Args:
             value: If given, uses this value instead of the entered one.
                    For boolean prompts, "yes"/"no" are accepted as value.
+            save: Save the value to the config.
         """
+        assert self._prompt is not None
         question = self._prompt.question
+
         try:
-            done = self._prompt.accept(value)
+            done = self._prompt.accept(value, save=save)
         except Error as e:
             raise cmdutils.CommandError(str(e))
+
         if done:
             message.global_bridge.prompt_done.emit(self._prompt.KEY_MODE)
             question.done()
@@ -405,6 +422,7 @@ class PromptContainer(QWidget):
                      cmdline.
             pdfjs: Open the download via PDF.js.
         """
+        assert self._prompt is not None
         try:
             self._prompt.download_open(cmdline, pdfjs=pdfjs)
         except UnsupportedOperationError:
@@ -419,6 +437,7 @@ class PromptContainer(QWidget):
         Args:
             which: 'next', 'prev'
         """
+        assert self._prompt is not None
         try:
             self._prompt.item_focus(which)
         except UnsupportedOperationError:
@@ -433,6 +452,7 @@ class PromptContainer(QWidget):
         Args:
             sel: Use the primary selection instead of the clipboard.
         """
+        assert self._prompt is not None
         question = self._prompt.question
         if question.url is None:
             message.error('No URL found.')
@@ -536,7 +556,12 @@ class _BasePrompt(QWidget):
 
         self._vbox.addLayout(self._key_grid)
 
-    def accept(self, value=None):
+    def _check_save_support(self, save):
+        if save:
+            raise UnsupportedOperationError("Saving answers is only possible "
+                                            "with yes/no prompts.")
+
+    def accept(self, value=None, save=False):
         raise NotImplementedError
 
     def download_open(self, cmdline, pdfjs):
@@ -568,7 +593,8 @@ class LineEditPrompt(_BasePrompt):
         self.setFocusProxy(self._lineedit)
         self._init_key_label()
 
-    def accept(self, value=None):
+    def accept(self, value=None, save=False):
+        self._check_save_support(save)
         text = value if value is not None else self._lineedit.text()
         self.question.answer = text
         return True
@@ -681,10 +707,11 @@ class FilenamePrompt(_BasePrompt):
         # Nothing selected initially
         self._file_view.setCurrentIndex(QModelIndex())
         # The model needs to be sorted so we get the correct first/last index
-        self._file_model.directoryLoaded.connect(
+        self._file_model.directoryLoaded.connect(  # type: ignore
             lambda: self._file_model.sort(0))
 
-    def accept(self, value=None):
+    def accept(self, value=None, save=False):
+        self._check_save_support(save)
         text = value if value is not None else self._lineedit.text()
         text = downloads.transform_path(text)
         if text is None:
@@ -727,7 +754,9 @@ class FilenamePrompt(_BasePrompt):
         idx = self._do_completion(idx, which)
 
         selmodel.setCurrentIndex(
-            idx, QItemSelectionModel.ClearAndSelect | QItemSelectionModel.Rows)
+            idx,
+            QItemSelectionModel.ClearAndSelect |  # type: ignore
+            QItemSelectionModel.Rows)
         self._insert_path(idx, clicked=False)
 
     def _do_completion(self, idx, which):
@@ -752,10 +781,11 @@ class DownloadFilenamePrompt(FilenamePrompt):
 
     def __init__(self, question, parent=None):
         super().__init__(question, parent)
-        self._file_model.setFilter(QDir.AllDirs | QDir.Drives | QDir.NoDot)
+        self._file_model.setFilter(
+            QDir.AllDirs | QDir.Drives | QDir.NoDot)  # type: ignore
 
-    def accept(self, value=None):
-        done = super().accept(value)
+    def accept(self, value=None, save=False):
+        done = super().accept(value, save)
         answer = self.question.answer
         if answer is not None:
             self.question.answer = downloads.FileDownloadTarget(answer)
@@ -763,7 +793,8 @@ class DownloadFilenamePrompt(FilenamePrompt):
 
     def download_open(self, cmdline, pdfjs):
         if pdfjs:
-            target = downloads.PDFJSDownloadTarget()
+            target = downloads.PDFJSDownloadTarget(
+            )  # type: downloads._DownloadTarget
         else:
             target = downloads.OpenFileDownloadTarget(cmdline)
 
@@ -808,7 +839,8 @@ class AuthenticationPrompt(_BasePrompt):
         assert not question.default, question.default
         self.setFocusProxy(self._user_lineedit)
 
-    def accept(self, value=None):
+    def accept(self, value=None, save=False):
+        self._check_save_support(save)
         if value is not None:
             if ':' not in value:
                 raise Error("Value needs to be in the format "
@@ -851,7 +883,14 @@ class YesNoPrompt(_BasePrompt):
         self._init_texts(question)
         self._init_key_label()
 
-    def accept(self, value=None):
+    def _check_save_support(self, save):
+        if save and self.question.option is None:
+            raise Error("No setting available to save the answer for this "
+                        "question.")
+
+    def accept(self, value=None, save=False):
+        self._check_save_support(save)
+
         if value is None:
             if self.question.default is None:
                 raise Error("No default value was set for this question!")
@@ -862,14 +901,30 @@ class YesNoPrompt(_BasePrompt):
             self.question.answer = False
         else:
             raise Error("Invalid value {} - expected yes/no!".format(value))
+
+        if save:
+            opt = config.instance.get_opt(self.question.option)
+            assert isinstance(opt.typ, configtypes.Bool)
+            pattern = urlmatch.UrlPattern(self.question.url)
+
+            try:
+                config.instance.set_obj(opt.name, self.question.answer,
+                                        pattern=pattern, save_yaml=True)
+            except configexc.Error as e:
+                raise Error(str(e))
+
         return True
 
     def _allowed_commands(self):
-        cmds = [
-            ('prompt-accept yes', "Yes"),
-            ('prompt-accept no', "No"),
-            ('prompt-yank', "Yank URL"),
-        ]
+        cmds = []
+
+        cmds.append(('prompt-accept yes', "Yes"))
+        if self.question.option is not None:
+            cmds.append(('prompt-accept --save yes', "Always"))
+
+        cmds.append(('prompt-accept no', "No"))
+        if self.question.option is not None:
+            cmds.append(('prompt-accept --save no', "Never"))
 
         if self.question.default is not None:
             assert self.question.default in [True, False]
@@ -877,6 +932,7 @@ class YesNoPrompt(_BasePrompt):
             cmds.append(('prompt-accept', "Use default ({})".format(default)))
 
         cmds.append(('leave-mode', "Abort"))
+        cmds.append(('prompt-yank', "Yank URL"))
         return cmds
 
 
@@ -889,7 +945,8 @@ class AlertPrompt(_BasePrompt):
         self._init_texts(question)
         self._init_key_label()
 
-    def accept(self, value=None):
+    def accept(self, value=None, save=False):
+        self._check_save_support(save)
         if value is not None:
             raise Error("No value is permitted with alert prompts!")
         # Simply mark prompt as done without setting self.question.answer
@@ -903,6 +960,5 @@ def init():
     """Initialize global prompt objects."""
     global prompt_queue
     prompt_queue = PromptQueue()
-    objreg.register('prompt-queue', prompt_queue)  # for commands
-    message.global_bridge.ask_question.connect(
+    message.global_bridge.ask_question.connect(  # type: ignore
         prompt_queue.ask_question, Qt.DirectConnection)
