@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2017 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -19,8 +19,9 @@
 
 """The tab widget used for TabbedBrowser from browser.py."""
 
+import typing
 import functools
-import enum
+import contextlib
 
 import attr
 from PyQt5.QtCore import (pyqtSignal, pyqtSlot, Qt, QSize, QRect, QPoint,
@@ -31,12 +32,9 @@ from PyQt5.QtWidgets import (QTabWidget, QTabBar, QSizePolicy, QCommonStyle,
 from PyQt5.QtGui import QIcon, QPalette, QColor
 
 from qutebrowser.utils import qtutils, objreg, utils, usertypes, log
-from qutebrowser.config import config
-from qutebrowser.misc import objects
-
-
-PixelMetrics = enum.IntEnum('PixelMetrics', ['icon_padding'],
-                            start=QStyle.PM_CustomBase)
+from qutebrowser.config import config, stylesheet
+from qutebrowser.misc import objects, debugcachestats
+from qutebrowser.browser import browsertab
 
 
 class TabWidget(QTabWidget):
@@ -53,15 +51,19 @@ class TabWidget(QTabWidget):
     tab_index_changed = pyqtSignal(int, int)
     new_tab_requested = pyqtSignal('QUrl', bool, bool)
 
+    # Strings for controlling the mute/audible text
+    MUTE_STRING = '[M] '
+    AUDIBLE_STRING = '[A] '
+
     def __init__(self, win_id, parent=None):
         super().__init__(parent)
         bar = TabBar(win_id, self)
         self.setStyle(TabBarStyle())
         self.setTabBar(bar)
-        bar.tabCloseRequested.connect(self.tabCloseRequested)
-        bar.tabMoved.connect(functools.partial(
-            QTimer.singleShot, 0, self._update_tab_titles))
-        bar.currentChanged.connect(self._on_current_changed)
+        bar.tabCloseRequested.connect(self.tabCloseRequested)  # type: ignore
+        bar.tabMoved.connect(functools.partial(  # type: ignore
+            QTimer.singleShot, 0, self.update_tab_titles))
+        bar.currentChanged.connect(self._on_current_changed)  # type: ignore
         bar.new_tab_requested.connect(self._on_new_tab_requested)
         self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
         self.setDocumentMode(True)
@@ -103,14 +105,10 @@ class TabWidget(QTabWidget):
             tab: The tab to pin
             pinned: Pinned tab state to set.
         """
-        bar = self.tabBar()
         idx = self.indexOf(tab)
-
-        bar.set_tab_data(idx, 'pinned', pinned)
         tab.data.pinned = pinned
-        self._update_tab_title(idx)
-
-        bar.refresh()
+        self.update_tab_favicon(tab)
+        self.update_tab_title(idx)
 
     def tab_indicator_color(self, idx):
         """Get the tab indicator color for the given index."""
@@ -118,14 +116,20 @@ class TabWidget(QTabWidget):
 
     def set_page_title(self, idx, title):
         """Set the tab title user data."""
-        self.tabBar().set_tab_data(idx, 'page-title', title)
-        self._update_tab_title(idx)
+        tabbar = self.tabBar()
+
+        if config.cache['tabs.tooltips']:
+            # always show only plain title in tooltips
+            tabbar.setTabToolTip(idx, title)
+
+        tabbar.set_tab_data(idx, 'page-title', title)
+        self.update_tab_title(idx)
 
     def page_title(self, idx):
         """Get the tab title user data."""
         return self.tabBar().page_title(idx)
 
-    def _update_tab_title(self, idx, field=None):
+    def update_tab_title(self, idx, field=None):
         """Update the tab text for the given tab.
 
         Args:
@@ -135,36 +139,52 @@ class TabWidget(QTabWidget):
         """
         tab = self.widget(idx)
         if tab.data.pinned:
-            fmt = config.val.tabs.title.format_pinned
+            fmt = config.cache['tabs.title.format_pinned']
         else:
-            fmt = config.val.tabs.title.format
+            fmt = config.cache['tabs.title.format']
 
         if (field is not None and
                 (fmt is None or ('{' + field + '}') not in fmt)):
             return
 
         fields = self.get_tab_fields(idx)
-        fields['title'] = fields['title'].replace('&', '&&')
+        fields['current_title'] = fields['current_title'].replace('&', '&&')
         fields['index'] = idx + 1
 
         title = '' if fmt is None else fmt.format(**fields)
-        self.tabBar().setTabText(idx, title)
+        tabbar = self.tabBar()
+
+        # Only change the tab title if it changes, setting the tab title causes
+        # a size recalculation which is slow.
+        if tabbar.tabText(idx) != title:
+            tabbar.setTabText(idx, title)
 
     def get_tab_fields(self, idx):
         """Get the tab field data."""
         tab = self.widget(idx)
         if tab is None:
-            log.misc.debug("Got None-tab in get_tab_fields!")
+            log.misc.debug(  # type: ignore
+                "Got None-tab in get_tab_fields!")
 
         page_title = self.page_title(idx)
 
         fields = {}
         fields['id'] = tab.tab_id
-        fields['title'] = page_title
+        fields['current_title'] = page_title
         fields['title_sep'] = ' - ' if page_title else ''
         fields['perc_raw'] = tab.progress()
         fields['backend'] = objects.backend.name
-        fields['private'] = ' [Private Mode] ' if tab.private else ''
+        fields['private'] = ' [Private Mode] ' if tab.is_private else ''
+        try:
+            if tab.audio.is_muted():
+                fields['audio'] = TabWidget.MUTE_STRING
+            elif tab.audio.is_recently_audible():
+                fields['audio'] = TabWidget.AUDIBLE_STRING
+            else:
+                fields['audio'] = ''
+        except browsertab.WebTabError:
+            # Muting is only implemented with QtWebEngine
+            fields['audio'] = ''
 
         if tab.load_status() == usertypes.LoadStatus.loading:
             fields['perc'] = '[{}%] '.format(tab.progress())
@@ -195,20 +215,45 @@ class TabWidget(QTabWidget):
         fields['scroll_pos'] = scroll_pos
         return fields
 
-    def _update_tab_titles(self):
+    @contextlib.contextmanager
+    def _toggle_visibility(self):
+        """Toggle visibility while running.
+
+        Every single call to setTabText calls the size hinting functions for
+        every single tab, which are slow. Since we know we are updating all
+        the tab's titles, we can delay this processing by making the tab
+        non-visible. To avoid flickering, disable repaint updates whlie we
+        work.
+        """
+        bar = self.tabBar()
+        toggle = (self.count() > 10 and
+                  not bar.drag_in_progress and
+                  bar.isVisible())
+        if toggle:
+            bar.setUpdatesEnabled(False)
+            bar.setVisible(False)
+
+        yield
+
+        if toggle:
+            bar.setVisible(True)
+            bar.setUpdatesEnabled(True)
+
+    def update_tab_titles(self):
         """Update all texts."""
-        for idx in range(self.count()):
-            self._update_tab_title(idx)
+        with self._toggle_visibility():
+            for idx in range(self.count()):
+                self.update_tab_title(idx)
 
     def tabInserted(self, idx):
         """Update titles when a tab was inserted."""
         super().tabInserted(idx)
-        self._update_tab_titles()
+        self.update_tab_titles()
 
     def tabRemoved(self, idx):
         """Update titles when a tab was removed."""
         super().tabRemoved(idx)
-        self._update_tab_titles()
+        self.update_tab_titles()
 
     def addTab(self, page, icon_or_text, text_or_empty=None):
         """Override addTab to use our own text setting logic.
@@ -228,7 +273,6 @@ class TabWidget(QTabWidget):
             The index of the newly added tab.
         """
         if text_or_empty is None:
-            icon = None
             text = icon_or_text
             new_idx = super().addTab(page, '')
         else:
@@ -258,7 +302,6 @@ class TabWidget(QTabWidget):
             The index of the newly added tab.
         """
         if text_or_empty is None:
-            icon = None
             text = icon_or_text
             new_idx = super().insertTab(idx, page, '')
         else:
@@ -287,13 +330,38 @@ class TabWidget(QTabWidget):
         """
         tab = self.widget(idx)
         if tab is None:
-            url = QUrl()
+            url = QUrl()  # type: ignore
         else:
             url = tab.url()
         # It's possible for url to be invalid, but the caller will handle that.
         qtutils.ensure_valid(url)
         return url
 
+    def update_tab_favicon(self, tab: QWidget) -> None:
+        """Update favicon of the given tab."""
+        idx = self.indexOf(tab)
+
+        if tab.data.should_show_icon():
+            self.setTabIcon(idx, tab.icon())
+            if config.val.tabs.tabs_are_windows:
+                self.window().setWindowIcon(tab.icon())
+        else:
+            self.setTabIcon(idx, QIcon())
+            if config.val.tabs.tabs_are_windows:
+                self.window().setWindowIcon(self.window().windowIcon())
+
+    def setTabIcon(self, idx: int, icon: QIcon):
+        """Always show tab icons for pinned tabs in some circumstances."""
+        tab = typing.cast(typing.Optional[browsertab.AbstractTab],
+                          self.widget(idx))
+        if (icon.isNull() and
+                config.cache['tabs.favicons.show'] != 'never' and
+                config.cache['tabs.pinned.shrink'] and
+                not self.tabBar().vertical and
+                tab is not None and tab.data.pinned):
+            icon = self.style().standardIcon(QStyle.SP_FileIcon)
+        super().setTabIcon(idx, icon)
+
 
 class TabBar(QTabBar):
 
@@ -313,6 +381,12 @@ class TabBar(QTabBar):
         new_tab_requested: Emitted when a new tab is requested.
     """
 
+    STYLESHEET = """
+        TabBar {
+            background-color: {{ conf.colors.tabs.bar.bg }};
+        }
+    """
+
     new_tab_requested = pyqtSignal()
 
     def __init__(self, win_id, parent=None):
@@ -327,7 +401,8 @@ class TabBar(QTabBar):
         self._auto_hide_timer.timeout.connect(self.maybe_hide)
         self._on_show_switching_delay_changed()
         self.setAutoFillBackground(True)
-        self._set_colors()
+        self.drag_in_progress = False
+        stylesheet.set_register(self)
         QTimer.singleShot(0, self.maybe_hide)
 
     def __repr__(self):
@@ -338,13 +413,11 @@ class TabBar(QTabBar):
         return self.parent().currentWidget()
 
     @pyqtSlot(str)
-    def _on_config_changed(self, option: str):
+    def _on_config_changed(self, option: str) -> None:
         if option == 'fonts.tabs':
             self._set_font()
         elif option == 'tabs.favicons.scale':
             self._set_icon_size()
-        elif option == 'colors.tabs.bar.bg':
-            self._set_colors()
         elif option == 'tabs.show_switching_delay':
             self._on_show_switching_delay_changed()
         elif option == 'tabs.show':
@@ -353,11 +426,14 @@ class TabBar(QTabBar):
         if option.startswith('colors.tabs.'):
             self.update()
 
-        # Clear _minimum_tab_size_hint_helper cache when appropriate
+        # Clear tab size caches when appropriate
         if option in ["tabs.indicator.padding",
                       "tabs.padding",
-                      "tabs.indicator.width"]:
+                      "tabs.indicator.width",
+                      "tabs.min_width",
+                      "tabs.pinned.shrink"]:
             self._minimum_tab_size_hint_helper.cache_clear()
+            self._minimum_tab_height.cache_clear()
 
     def _on_show_switching_delay_changed(self):
         """Set timer interval when tabs.show_switching_delay got changed."""
@@ -434,21 +510,24 @@ class TabBar(QTabBar):
         self._set_icon_size()
         # clear tab size cache
         self._minimum_tab_size_hint_helper.cache_clear()
+        self._minimum_tab_height.cache_clear()
 
     def _set_icon_size(self):
         """Set the tab bar favicon size."""
         size = self.fontMetrics().height() - 2
-        size *= config.val.tabs.favicons.scale
+        size = int(size * config.val.tabs.favicons.scale)
         self.setIconSize(QSize(size, size))
 
-    def _set_colors(self):
-        """Set the tab bar colors."""
-        p = self.palette()
-        p.setColor(QPalette.Window, config.val.colors.tabs.bar.bg)
-        self.setPalette(p)
+    def mouseReleaseEvent(self, e):
+        """Override mouseReleaseEvent to know when drags stop."""
+        self.drag_in_progress = False
+        super().mouseReleaseEvent(e)
 
     def mousePressEvent(self, e):
-        """Override mousePressEvent to close tabs if configured."""
+        """Override mousePressEvent to close tabs if configured.
+
+        Also keep track of if we are currently in a drag."""
+        self.drag_in_progress = True
         button = config.val.tabs.close_mouse_button
         if (e.button() == Qt.RightButton and button == 'right' or
                 e.button() == Qt.MiddleButton and button == 'middle'):
@@ -465,36 +544,42 @@ class TabBar(QTabBar):
                     idx = self.currentIndex()
                 elif action == 'close-last':
                     idx = self.count() - 1
-            self.tabCloseRequested.emit(idx)
+            self.tabCloseRequested.emit(idx)  # type: ignore
             return
         super().mousePressEvent(e)
 
-    def minimumTabSizeHint(self, index, ellipsis: bool = True) -> QSize:
+    def minimumTabSizeHint(self, index: int, ellipsis: bool = True) -> QSize:
         """Set the minimum tab size to indicator/icon/... text.
 
         Args:
             index: The index of the tab to get a size hint for.
             ellipsis: Whether to use ellipsis to calculate width
-                     instead of the tab's text.
+                      instead of the tab's text.
+                      Forced to False for pinned tabs.
         Return:
             A QSize of the smallest tab size we can make.
         """
         icon = self.tabIcon(index)
-        icon_padding = self.style().pixelMetric(PixelMetrics.icon_padding,
-                                                None, self)
         if icon.isNull():
             icon_width = 0
         else:
-            icon_width = min(icon.actualSize(self.iconSize()).width(),
-                             self.iconSize().width()) + icon_padding
-        return self._minimum_tab_size_hint_helper(self.tabText(index),
-                                                  icon_width,
-                                                  ellipsis)
+            icon_width = min(
+                icon.actualSize(self.iconSize()).width(),
+                self.iconSize().width()) + TabBarStyle.ICON_PADDING
 
+        pinned = self._tab_pinned(index)
+        if not self.vertical and pinned and config.val.tabs.pinned.shrink:
+            # Never consider ellipsis an option for horizontal pinned tabs
+            ellipsis = False
+        return self._minimum_tab_size_hint_helper(self.tabText(index),
+                                                  icon_width, ellipsis,
+                                                  pinned)
+
+    @debugcachestats.register(name='tab width cache')
     @functools.lru_cache(maxsize=2**9)
     def _minimum_tab_size_hint_helper(self, tab_text: str,
                                       icon_width: int,
-                                      ellipsis: bool) -> QSize:
+                                      ellipsis: bool, pinned: bool) -> QSize:
         """Helper function to cache tab results.
 
         Config values accessed in here should be added to _on_config_changed to
@@ -508,36 +593,42 @@ class TabBar(QTabBar):
             return self.fontMetrics().size(Qt.TextShowMnemonic, text).width()
         text_width = min(_text_to_width(text),
                          _text_to_width(tab_text))
-        padding = config.val.tabs.padding
-        indicator_width = config.val.tabs.indicator.width
-        indicator_padding = config.val.tabs.indicator.padding
+        padding = config.cache['tabs.padding']
+        indicator_width = config.cache['tabs.indicator.width']
+        indicator_padding = config.cache['tabs.indicator.padding']
         padding_h = padding.left + padding.right
+
         # Only add padding if indicator exists
         if indicator_width != 0:
             padding_h += indicator_padding.left + indicator_padding.right
-        padding_v = padding.top + padding.bottom
-        height = self.fontMetrics().height() + padding_v
+        height = self._minimum_tab_height()
         width = (text_width + icon_width +
                  padding_h + indicator_width)
+        min_width = config.cache['tabs.min_width']
+        if (not self.vertical and min_width > 0 and
+                not pinned or not config.cache['tabs.pinned.shrink']):
+            width = max(min_width, width)
         return QSize(width, height)
 
-    def _pinned_statistics(self) -> (int, int):
-        """Get the number of pinned tabs and the total width of pinned tabs."""
-        pinned_list = [idx for idx in range(self.count())
-                       if self._tab_pinned(idx)]
-        pinned_count = len(pinned_list)
-        pinned_width = sum(self.minimumTabSizeHint(idx, ellipsis=False).width()
-                           for idx in pinned_list)
-        return (pinned_count, pinned_width)
+    @functools.lru_cache(maxsize=1)
+    def _minimum_tab_height(self):
+        padding = config.cache['tabs.padding']
+        return self.fontMetrics().height() + padding.top + padding.bottom
 
     def _tab_pinned(self, index: int) -> bool:
         """Return True if tab is pinned."""
-        try:
-            return self.tab_data(index, 'pinned')
-        except KeyError:
-            return False
+        if not 0 <= index < self.count():
+            raise IndexError("Tab index ({}) out of range ({})!".format(
+                index, self.count()))
 
-    def tabSizeHint(self, index: int):
+        widget = self.parent().widget(index)
+        if widget is None:
+            # This could happen when Qt calls tabSizeHint while initializing
+            # tabs.
+            return False
+        return widget.data.pinned
+
+    def tabSizeHint(self, index: int) -> QSize:
         """Override tabSizeHint to customize qb's tab size.
 
         https://wiki.python.org/moin/PyQt/Customising%20tab%20bars
@@ -548,10 +639,15 @@ class TabBar(QTabBar):
         Return:
             A QSize.
         """
-        minimum_size = self.minimumTabSizeHint(index)
-        height = minimum_size.height()
+        if self.count() == 0:
+            # This happens on startup on macOS.
+            # We return it directly rather than setting `size' because we don't
+            # want to ensure it's valid in this special case.
+            return QSize()
+
+        height = self._minimum_tab_height()
         if self.vertical:
-            confwidth = str(config.val.tabs.width)
+            confwidth = str(config.cache['tabs.width'])
             if confwidth.endswith('%'):
                 main_window = objreg.get('main-window', scope='window',
                                          window=self._win_id)
@@ -559,67 +655,50 @@ class TabBar(QTabBar):
                 width = main_window.width() * perc / 100
             else:
                 width = int(confwidth)
-            size = QSize(max(minimum_size.width(), width), height)
-        elif self.count() == 0:
-            # This happens on startup on macOS.
-            # We return it directly rather than setting `size' because we don't
-            # want to ensure it's valid in this special case.
-            return QSize()
+            size = QSize(width, height)
         else:
-            if config.val.tabs.pinned.shrink:
-                pinned = self._tab_pinned(index)
-                pinned_count, pinned_width = self._pinned_statistics()
-            else:
-                pinned = False
-                pinned_count, pinned_width = 0, 0
-            no_pinned_count = self.count() - pinned_count
-            no_pinned_width = self.width() - pinned_width
-
-            if pinned:
+            if config.cache['tabs.pinned.shrink'] and self._tab_pinned(index):
                 # Give pinned tabs the minimum size they need to display their
                 # titles, let Qt handle scaling it down if we get too small.
                 width = self.minimumTabSizeHint(index, ellipsis=False).width()
             else:
-                width = no_pinned_width / no_pinned_count
-
-            # If no_pinned_width is not divisible by no_pinned_count, add a
-            # pixel to some tabs so that there is no ugly leftover space.
-            if (no_pinned_count > 0 and
-                    index < no_pinned_width % no_pinned_count):
-                width += 1
-
-            # If we don't have enough space, we return the minimum size so we
-            # get scroll buttons as soon as needed.
-            width = max(width, minimum_size.width())
-
+                # Request as much space as possible so we fill the tabbar, let
+                # Qt shrink us down. If for some reason (tests, bugs)
+                # self.width() gives 0, use a sane min of 10 px
+                width = max(self.width(), 10)
+                max_width = config.cache['tabs.max_width']
+                if max_width > 0:
+                    width = min(max_width, width)
             size = QSize(width, height)
         qtutils.ensure_valid(size)
         return size
 
-    def paintEvent(self, _e):
+    def paintEvent(self, event):
         """Override paintEvent to draw the tabs like we want to."""
         p = QStylePainter(self)
         selected = self.currentIndex()
         for idx in range(self.count()):
+            if not event.region().intersects(self.tabRect(idx)):
+                # Don't repaint if we are outside the requested region
+                continue
+
             tab = QStyleOptionTab()
             self.initStyleOption(tab, idx)
 
-            # pylint: disable=bad-config-option
-            setting = config.val.colors.tabs
-            # pylint: enable=bad-config-option
+            setting = 'colors.tabs'
+            if self._tab_pinned(idx):
+                setting += '.pinned'
             if idx == selected:
-                setting = setting.selected
-            setting = setting.odd if (idx + 1) % 2 else setting.even
+                setting += '.selected'
+            setting += '.odd' if (idx + 1) % 2 else '.even'
 
-            tab.palette.setColor(QPalette.Window, setting.bg)
-            tab.palette.setColor(QPalette.WindowText, setting.fg)
+            tab.palette.setColor(QPalette.Window,
+                                 config.cache[setting + '.bg'])
+            tab.palette.setColor(QPalette.WindowText,
+                                 config.cache[setting + '.fg'])
 
             indicator_color = self.tab_indicator_color(idx)
             tab.palette.setColor(QPalette.Base, indicator_color)
-            if tab.rect.right() < 0 or tab.rect.left() > self.width():
-                # Don't bother drawing a tab if the entire tab is outside of
-                # the visible tab bar.
-                continue
             p.drawControl(QStyle.CE_TabBarTab, tab)
 
     def tabInserted(self, idx):
@@ -677,6 +756,8 @@ class TabBarStyle(QCommonStyle):
     https://code.google.com/p/makehuman/source/browse/trunk/makehuman/lib/qtgui.py
     """
 
+    ICON_PADDING = 4
+
     def __init__(self):
         """Initialize all functions we're not overriding.
 
@@ -757,11 +838,15 @@ class TabBarStyle(QCommonStyle):
         elif element == QStyle.CE_TabBarTabLabel:
             if not opt.icon.isNull() and layouts.icon.isValid():
                 self._draw_icon(layouts, opt, p)
-            alignment = (config.val.tabs.title.alignment |
+            alignment = (config.cache['tabs.title.alignment'] |
                          Qt.AlignVCenter | Qt.TextHideMnemonic)
-            self._style.drawItemText(p, layouts.text, alignment, opt.palette,
-                                     opt.state & QStyle.State_Enabled,
-                                     opt.text, QPalette.WindowText)
+            self._style.drawItemText(p,
+                                     layouts.text,
+                                     int(alignment),
+                                     opt.palette,
+                                     bool(opt.state & QStyle.State_Enabled),
+                                     opt.text,
+                                     QPalette.WindowText)
         else:
             raise ValueError("Invalid element {!r}".format(element))
 
@@ -782,8 +867,6 @@ class TabBarStyle(QCommonStyle):
                       QStyle.PM_TabBarTabVSpace,
                       QStyle.PM_TabBarScrollButtonWidth]:
             return 0
-        elif metric == PixelMetrics.icon_padding:
-            return 4
         else:
             return self._style.pixelMetric(metric, option, widget)
 
@@ -830,8 +913,8 @@ class TabBarStyle(QCommonStyle):
         Return:
             A Layout object with two QRects.
         """
-        padding = config.val.tabs.padding
-        indicator_padding = config.val.tabs.indicator.padding
+        padding = config.cache['tabs.padding']
+        indicator_padding = config.cache['tabs.indicator.padding']
 
         text_rect = QRect(opt.rect)
         if not text_rect.isValid():
@@ -842,7 +925,7 @@ class TabBarStyle(QCommonStyle):
         text_rect.adjust(padding.left, padding.top, -padding.right,
                          -padding.bottom)
 
-        indicator_width = config.val.tabs.indicator.width
+        indicator_width = config.cache['tabs.indicator.width']
         if indicator_width == 0:
             indicator_rect = QRect()
         else:
@@ -859,8 +942,8 @@ class TabBarStyle(QCommonStyle):
 
         icon_rect = self._get_icon_rect(opt, text_rect)
         if icon_rect.isValid():
-            icon_padding = self.pixelMetric(PixelMetrics.icon_padding, opt)
-            text_rect.adjust(icon_rect.width() + icon_padding, 0, 0, 0)
+            text_rect.adjust(
+                icon_rect.width() + TabBarStyle.ICON_PADDING, 0, 0, 0)
 
         text_rect = self._style.visualRect(opt.direction, opt.rect, text_rect)
         return Layouts(text=text_rect, icon=icon_rect,
@@ -885,9 +968,9 @@ class TabBarStyle(QCommonStyle):
         icon_state = (QIcon.On if opt.state & QStyle.State_Selected
                       else QIcon.Off)
         # reserve space for favicon when tab bar is vertical (issue #1968)
-        position = config.val.tabs.position
+        position = config.cache['tabs.position']
         if (position in [QTabWidget.East, QTabWidget.West] and
-                config.val.tabs.favicons.show):
+                config.cache['tabs.favicons.show'] != 'never'):
             tab_icon_size = icon_size
         else:
             actual_size = opt.icon.actualSize(icon_size, icon_mode, icon_state)
@@ -895,7 +978,7 @@ class TabBarStyle(QCommonStyle):
                 min(actual_size.width(), icon_size.width()),
                 min(actual_size.height(), icon_size.height()))
 
-        icon_top = text_rect.center().y() + 1 - tab_icon_size.height() / 2
+        icon_top = text_rect.center().y() + 1 - tab_icon_size.height() // 2
         icon_rect = QRect(QPoint(text_rect.left(), icon_top), tab_icon_size)
         icon_rect = self._style.visualRect(opt.direction, opt.rect, icon_rect)
         return icon_rect
