@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -55,7 +55,7 @@ class PyPIVersionClient(QObject):
             self._client = httpclient.HTTPClient(self)
         else:
             self._client = client
-        self._client.error.connect(self.error)
+        self._client.error.connect(self.error)  # type: ignore
         self._client.success.connect(self.on_client_success)
 
     def get_version(self, package='qutebrowser'):
