@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2018 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -28,6 +28,7 @@ import fnmatch
 import traceback
 import datetime
 import enum
+import typing
 
 import pkg_resources
 from PyQt5.QtCore import pyqtSlot, Qt, QSize
@@ -36,14 +37,19 @@ from PyQt5.QtWidgets import (QDialog, QLabel, QTextEdit, QPushButton,
                              QDialogButtonBox, QApplication, QMessageBox)
 
 import qutebrowser
-from qutebrowser.utils import version, log, utils, objreg
+from qutebrowser.utils import version, log, utils
 from qutebrowser.misc import (miscwidgets, autoupdate, msgbox, httpclient,
                               pastebin)
 from qutebrowser.config import config, configfiles
+from qutebrowser.browser import history
 
 
-Result = enum.IntEnum('Result', ['restore', 'no_restore'],
-                      start=QDialog.Accepted + 1)
+class Result(enum.IntEnum):
+
+    """The result code returned by the crash dialog."""
+
+    restore = QDialog.Accepted + 1
+    no_restore = QDialog.Accepted + 2
 
 
 def parse_fatal_stacktrace(text):
@@ -113,11 +119,8 @@ class _CrashDialog(QDialog):
         super().__init__(parent)
         # We don't set WA_DeleteOnClose here as on an exception, we'll get
         # closed anyways, and it only could have unintended side-effects.
-        self._crash_info = []
+        self._crash_info = []  # type: typing.List[typing.Tuple[str, str]]
         self._btn_box = None
-        self._btn_report = None
-        self._btn_cancel = None
-        self._lbl = None
         self._paste_text = None
         self.setWindowTitle("Whoops!")
         self.resize(QSize(640, 600))
@@ -131,26 +134,25 @@ class _CrashDialog(QDialog):
 
         info = QLabel("What were you doing when this crash/bug happened?")
         self._vbox.addWidget(info)
-        self._info = QTextEdit(tabChangesFocus=True, acceptRichText=False)
+        self._info = QTextEdit()
+        self._info.setTabChangesFocus(True)
+        self._info.setAcceptRichText(False)
         self._info.setPlaceholderText("- Opened http://www.example.com/\n"
                                       "- Switched tabs\n"
                                       "- etc...")
         self._vbox.addWidget(self._info, 5)
 
         self._vbox.addSpacing(15)
-        self._debug_log = QTextEdit(tabChangesFocus=True, acceptRichText=False,
-                                    lineWrapMode=QTextEdit.NoWrap)
+        self._debug_log = QTextEdit()
+        self._debug_log.setTabChangesFocus(True)
+        self._debug_log.setAcceptRichText(False)
+        self._debug_log.setLineWrapMode(QTextEdit.NoWrap)
         self._debug_log.hide()
-        info = QLabel("<i>You can edit the log below to remove sensitive "
-                      "information.</i>", wordWrap=True)
-        info.hide()
         self._fold = miscwidgets.DetailFold("Show log", self)
         self._fold.toggled.connect(self._debug_log.setVisible)
-        self._fold.toggled.connect(info.setVisible)
         if debug:
             self._fold.toggle()
         self._vbox.addWidget(self._fold)
-        self._vbox.addWidget(info)
         self._vbox.addWidget(self._debug_log, 10)
         self._vbox.addSpacing(15)
 
@@ -170,10 +172,12 @@ class _CrashDialog(QDialog):
         """Initialize the widget asking for contact info."""
         contact = QLabel("I'd like to be able to follow up with you, to keep "
                          "you posted on the status of this crash and get more "
-                         "information if I need it - how can I contact you?",
-                         wordWrap=True)
+                         "information if I need it - how can I contact you?")
+        contact.setWordWrap(True)
         self._vbox.addWidget(contact)
-        self._contact = QTextEdit(tabChangesFocus=True, acceptRichText=False)
+        self._contact = QTextEdit()
+        self._contact.setTabChangesFocus(True)
+        self._contact.setAcceptRichText(False)
         try:
             try:
                 info = configfiles.state['general']['contact-info']
@@ -191,32 +195,38 @@ class _CrashDialog(QDialog):
 
         Should be extended by subclasses to set the actual text.
         """
-        self._lbl = QLabel(wordWrap=True, openExternalLinks=True,
-                           textInteractionFlags=Qt.LinksAccessibleByMouse)
+        self._lbl = QLabel()
+        self._lbl.setWordWrap(True)
+        self._lbl.setOpenExternalLinks(True)
+        self._lbl.setTextInteractionFlags(Qt.LinksAccessibleByMouse)
         self._vbox.addWidget(self._lbl)
 
     def _init_checkboxes(self):
         """Initialize the checkboxes."""
-        pass
 
     def _init_buttons(self):
         """Initialize the buttons."""
         self._btn_box = QDialogButtonBox()
         self._vbox.addWidget(self._btn_box)
 
-        self._btn_report = QPushButton("Report", default=True)
+        self._btn_report = QPushButton("Report")
+        self._btn_report.setDefault(True)
         self._btn_report.clicked.connect(self.on_report_clicked)
         self._btn_box.addButton(self._btn_report, QDialogButtonBox.AcceptRole)
 
-        self._btn_cancel = QPushButton("Don't report", autoDefault=False)
+        self._btn_cancel = QPushButton("Don't report")
+        self._btn_cancel.setAutoDefault(False)
         self._btn_cancel.clicked.connect(self.finish)
         self._btn_box.addButton(self._btn_cancel, QDialogButtonBox.RejectRole)
 
     def _init_info_text(self):
         """Add an info text encouraging the user to report crashes."""
-        info_label = QLabel("<br/><b>Note that without your help, I can't fix "
-                            "the bug you encountered.</b>",
-                            wordWrap=True)
+        info_label = QLabel("<br/>There is currently a big backlog of crash "
+                            "reports. Thus, it might take a while until your "
+                            "report is seen.<br/>A new tool allowing for more "
+                            "automation will fix this, but is not ready yet "
+                            "at this point.")
+        info_label.setWordWrap(True)
         self._vbox.addWidget(info_label)
 
     def _gather_crash_info(self):
@@ -335,7 +345,7 @@ class _CrashDialog(QDialog):
             text: The paste text to show.
         """
         error_dlg = ReportErrorDialog(text, self._paste_text, self)
-        error_dlg.finished.connect(self.finish)
+        error_dlg.finished.connect(self.finish)  # type: ignore
         error_dlg.show()
 
     @pyqtSlot(str)
@@ -391,8 +401,6 @@ class ExceptionCrashDialog(_CrashDialog):
     """
 
     def __init__(self, debug, pages, cmdhist, exc, qobjects, parent=None):
-        self._chk_log = None
-        self._chk_restore = None
         super().__init__(debug, parent)
         self._pages = pages
         self._cmdhist = cmdhist
@@ -412,8 +420,8 @@ class ExceptionCrashDialog(_CrashDialog):
         self._chk_restore = QCheckBox("Restore open pages")
         self._chk_restore.setChecked(True)
         self._vbox.addWidget(self._chk_restore)
-        self._chk_log = QCheckBox("Include a debug log in the report",
-                                  checked=True)
+        self._chk_log = QCheckBox("Include a debug log in the report")
+        self._chk_log.setChecked(True)
         try:
             if config.val.content.private_browsing:
                 self._chk_log.setChecked(False)
@@ -421,10 +429,12 @@ class ExceptionCrashDialog(_CrashDialog):
             log.misc.exception("Error while checking private browsing mode")
         self._chk_log.toggled.connect(self._set_crash_info)
         self._vbox.addWidget(self._chk_log)
-        info_label = QLabel("<i>This makes it a lot easier to diagnose the "
-                            "crash, but it might contain sensitive "
-                            "information such as which pages you visited "
-                            "or keyboard input.</i>", wordWrap=True)
+        info_label = QLabel("This makes it a lot easier to diagnose the "
+                            "crash.<br/><b>Note that the log might contain "
+                            "sensitive information such as which pages you "
+                            "visited or keyboard input.</b><br/>You can show "
+                            "and edit the log above.")
+        info_label.setWordWrap(True)
         self._vbox.addWidget(info_label)
 
     def _get_error_type(self):
@@ -447,8 +457,10 @@ class ExceptionCrashDialog(_CrashDialog):
                 ("Objects", self._qobjects),
             ]
             try:
-                self._crash_info.append(
-                    ("Debug log", log.ram_handler.dump_log()))
+                text = "Log output was disabled."
+                if log.ram_handler is not None:
+                    text = log.ram_handler.dump_log()
+                self._crash_info.append(("Debug log", text))
             except Exception:
                 self._crash_info.append(("Debug log", traceback.format_exc()))
 
@@ -474,7 +486,6 @@ class FatalCrashDialog(_CrashDialog):
     """
 
     def __init__(self, debug, text, parent=None):
-        self._chk_history = None
         super().__init__(debug, parent)
         self._log = text
         self.setAttribute(Qt.WA_DeleteOnClose)
@@ -496,7 +507,7 @@ class FatalCrashDialog(_CrashDialog):
         text = ("<b>qutebrowser was restarted after a fatal crash.</b><br/>"
                 "<br/>Note: Crash reports for fatal crashes sometimes don't "
                 "contain the information necessary to fix an issue. Please "
-                "follow the steps in <a href='https://github.com/The-Compiler/"
+                "follow the steps in <a href='https://github.com/qutebrowser/"
                 "qutebrowser/blob/master/doc/stacktrace.asciidoc'>"
                 "stacktrace.asciidoc</a> to submit a stacktrace.<br/>")
         self._lbl.setText(text)
@@ -505,8 +516,8 @@ class FatalCrashDialog(_CrashDialog):
         """Add checkboxes to the dialog."""
         super()._init_checkboxes()
         self._chk_history = QCheckBox("Include a history of the last "
-                                      "accessed pages in the report.",
-                                      checked=True)
+                                      "accessed pages in the report.")
+        self._chk_history.setChecked(True)
         try:
             if config.val.content.private_browsing:
                 self._chk_history.setChecked(False)
@@ -520,11 +531,14 @@ class FatalCrashDialog(_CrashDialog):
         super()._gather_crash_info()
         if self._chk_history.isChecked():
             try:
-                history = objreg.get('web-history').get_recent()
-                self._crash_info.append(("History",
-                                         '\n'.join(str(e) for e in history)))
+                if history.web_history is None:
+                    history_data = '<unavailable>'  # type: ignore
+                else:
+                    history_data = '\n'.join(str(e) for e in
+                                             history.web_history.get_recent())
             except Exception:
-                self._crash_info.append(("History", traceback.format_exc()))
+                history_data = traceback.format_exc()
+            self._crash_info.append(("History", history_data))
 
     @pyqtSlot()
     def on_report_clicked(self):
@@ -569,7 +583,6 @@ class ReportDialog(_CrashDialog):
 
     def _init_info_text(self):
         """We don't want an info text as the user wanted to report."""
-        pass
 
     def _get_error_type(self):
         return 'report'
@@ -583,7 +596,10 @@ class ReportDialog(_CrashDialog):
             ("Objects", self._qobjects),
         ]
         try:
-            self._crash_info.append(("Debug log", log.ram_handler.dump_log()))
+            text = "Log output was disabled."
+            if log.ram_handler is not None:
+                text = log.ram_handler.dump_log()
+            self._crash_info.append(("Debug log", text))
         except Exception:
             self._crash_info.append(("Debug log", traceback.format_exc()))
 
@@ -602,8 +618,10 @@ class ReportErrorDialog(QDialog):
                        "crash@qutebrowser.org</a> - Thanks!".format(
                            html.escape(exc_text)))
         vbox.addWidget(label)
-        txt = QTextEdit(readOnly=True, tabChangesFocus=True,
-                        acceptRichText=False)
+        txt = QTextEdit()
+        txt.setReadOnly(True)
+        txt.setTabChangesFocus(True)
+        txt.setAcceptRichText(False)
         txt.setText(text)
         txt.selectAll()
         vbox.addWidget(txt)
@@ -611,7 +629,7 @@ class ReportErrorDialog(QDialog):
         hbox = QHBoxLayout()
         hbox.addStretch()
         btn = QPushButton("Close")
-        btn.clicked.connect(self.close)
+        btn.clicked.connect(self.close)  # type: ignore
         hbox.addWidget(btn)
         vbox.addLayout(hbox)
 
