@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2017 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -22,7 +22,8 @@
 import os
 import tempfile
 
-from PyQt5.QtCore import pyqtSignal, pyqtSlot, QObject, QProcess
+from PyQt5.QtCore import (pyqtSignal, pyqtSlot, QObject, QProcess,
+                          QFileSystemWatcher)
 
 from qutebrowser.config import config
 from qutebrowser.utils import message, log
@@ -39,23 +40,41 @@ class ExternalEditor(QObject):
         _remove_file: Whether the file should be removed when the editor is
                       closed.
         _proc: The GUIProcess of the editor.
+        _watcher: A QFileSystemWatcher to watch the edited file for changes.
+                  Only set if watch=True.
+        _content: The last-saved text of the editor.
+
+    Signals:
+        file_updated: The text in the edited file was updated.
+                      arg: The new text.
+        editing_finished: The editor process was closed.
     """
 
-    editing_finished = pyqtSignal(str)
+    file_updated = pyqtSignal(str)
+    editing_finished = pyqtSignal()
 
-    def __init__(self, parent=None):
+    def __init__(self, parent=None, watch=False):
         super().__init__(parent)
         self._filename = None
         self._proc = None
         self._remove_file = None
+        self._watcher = QFileSystemWatcher(parent=self) if watch else None
+        self._content = None
 
     def _cleanup(self):
         """Clean up temporary files after the editor closed."""
         assert self._remove_file is not None
+        if self._watcher is not None and self._watcher.files():
+            failed = self._watcher.removePaths(self._watcher.files())
+            if failed:
+                log.procs.error("Failed to unwatch paths: {}".format(failed))
+
         if self._filename is None or not self._remove_file:
             # Could not create initial file.
             return
 
+        assert self._proc is not None
+
         try:
             if self._proc.exit_status() != QProcess.CrashExit:
                 os.remove(self._filename)
@@ -65,7 +84,7 @@ class ExternalEditor(QObject):
             message.error("Failed to delete tempfile... ({})".format(e))
 
     @pyqtSlot(int, QProcess.ExitStatus)
-    def on_proc_closed(self, exitcode, exitstatus):
+    def _on_proc_closed(self, _exitcode, exitstatus):
         """Write the editor text into the form field and clean up tempfile.
 
         Callback for QProcess when the editor was closed.
@@ -75,25 +94,13 @@ class ExternalEditor(QObject):
             # No error/cleanup here, since we already handle this in
             # on_proc_error.
             return
-        try:
-            if exitcode != 0:
-                return
-            encoding = config.val.editor.encoding
-            try:
-                with open(self._filename, 'r', encoding=encoding) as f:
-                    text = f.read()
-            except OSError as e:
-                # NOTE: Do not replace this with "raise CommandError" as it's
-                # executed async.
-                message.error("Failed to read back edited file: {}".format(e))
-                return
-            log.procs.debug("Read back: {}".format(text))
-            self.editing_finished.emit(text)
-        finally:
-            self._cleanup()
+        # do a final read to make sure we don't miss the last signal
+        self._on_file_changed(self._filename)
+        self.editing_finished.emit()
+        self._cleanup()
 
     @pyqtSlot(QProcess.ProcessError)
-    def on_proc_error(self, _err):
+    def _on_proc_error(self, _err):
         self._cleanup()
 
     def edit(self, text, caret_position=None):
@@ -106,19 +113,7 @@ class ExternalEditor(QObject):
         if self._filename is not None:
             raise ValueError("Already editing a file!")
         try:
-            # Close while the external process is running, as otherwise systems
-            # with exclusive write access (e.g. Windows) may fail to update
-            # the file from the external editor, see
-            # https://github.com/qutebrowser/qutebrowser/issues/1767
-            with tempfile.NamedTemporaryFile(
-                    # pylint: disable=bad-continuation
-                    mode='w', prefix='qutebrowser-editor-',
-                    encoding=config.val.editor.encoding,
-                    delete=False) as fobj:
-                    # pylint: enable=bad-continuation
-                if text:
-                    fobj.write(text)
-                self._filename = fobj.name
+            self._filename = self._create_tempfile(text, 'qutebrowser-editor-')
         except OSError as e:
             message.error("Failed to create initial file: {}".format(e))
             return
@@ -128,6 +123,45 @@ class ExternalEditor(QObject):
         line, column = self._calc_line_and_column(text, caret_position)
         self._start_editor(line=line, column=column)
 
+    def backup(self):
+        """Create a backup if the content has changed from the original."""
+        if not self._content:
+            return
+        try:
+            fname = self._create_tempfile(self._content,
+                                          'qutebrowser-editor-backup-')
+            message.info('Editor backup at {}'.format(fname))
+        except OSError as e:
+            message.error('Failed to create editor backup: {}'.format(e))
+
+    def _create_tempfile(self, text, prefix):
+        # Close while the external process is running, as otherwise systems
+        # with exclusive write access (e.g. Windows) may fail to update
+        # the file from the external editor, see
+        # https://github.com/qutebrowser/qutebrowser/issues/1767
+        with tempfile.NamedTemporaryFile(
+                mode='w', prefix=prefix,
+                encoding=config.val.editor.encoding,
+                delete=False) as fobj:
+            if text:
+                fobj.write(text)
+            return fobj.name
+
+    @pyqtSlot(str)
+    def _on_file_changed(self, path):
+        try:
+            with open(path, 'r', encoding=config.val.editor.encoding) as f:
+                text = f.read()
+        except OSError as e:
+            # NOTE: Do not replace this with "raise CommandError" as it's
+            # executed async.
+            message.error("Failed to read back edited file: {}".format(e))
+            return
+        log.procs.debug("Read back: {}".format(text))
+        if self._content != text:
+            self._content = text
+            self.file_updated.emit(text)
+
     def edit_file(self, filename):
         """Edit the file with the given filename."""
         self._filename = filename
@@ -142,11 +176,20 @@ class ExternalEditor(QObject):
             column: the column number to pass to the editor
         """
         self._proc = guiprocess.GUIProcess(what='editor', parent=self)
-        self._proc.finished.connect(self.on_proc_closed)
-        self._proc.error.connect(self.on_proc_error)
+        self._proc.finished.connect(self._on_proc_closed)
+        self._proc.error.connect(self._on_proc_error)
         editor = config.val.editor.command
         executable = editor[0]
 
+        if self._watcher:
+            assert self._filename is not None
+            ok = self._watcher.addPath(self._filename)
+            if not ok:
+                log.procs.error("Failed to watch path: {}"
+                                .format(self._filename))
+            self._watcher.fileChanged.connect(  # type: ignore
+                self._on_file_changed)
+
         args = [self._sub_placeholder(arg, line, column) for arg in editor[1:]]
         log.procs.debug("Calling \"{}\" with args {}".format(executable, args))
         self._proc.start(executable, args)
