@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2016-2019 Ryan Roden-Corrent (rcorre) <ryan@rcorre.net>
+# Copyright 2016-2020 Ryan Roden-Corrent (rcorre) <ryan@rcorre.net>
 #
 # This file is part of qutebrowser.
 #
@@ -31,7 +31,7 @@ import re
 from PyQt5.QtWidgets import QLabel, QSizePolicy
 from PyQt5.QtCore import pyqtSlot, pyqtSignal, Qt
 
-from qutebrowser.config import config
+from qutebrowser.config import config, stylesheet
 from qutebrowser.utils import utils, usertypes
 from qutebrowser.misc import objects
 from qutebrowser.keyinput import keyutils
@@ -71,7 +71,8 @@ class KeyHintView(QLabel):
         self.hide()
         self._show_timer = usertypes.Timer(self, 'keyhint_show')
         self._show_timer.timeout.connect(self.show)
-        config.set_register_stylesheet(self)
+        self._show_timer.setSingleShot(True)
+        stylesheet.set_register(self)
 
     def __repr__(self):
         return utils.get_repr(self, win_id=self._win_id)
@@ -88,7 +89,10 @@ class KeyHintView(QLabel):
         Args:
             prefix: The current partial keystring.
         """
-        countstr, prefix = re.fullmatch(r'(\d*)(.*)', prefix).groups()
+        match = re.fullmatch(r'(\d*)(.*)', prefix)
+        assert match is not None, prefix
+
+        countstr, prefix = match.groups()
         if not prefix:
             self._show_timer.stop()
             self.hide()
