@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2017-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2017-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -23,10 +23,9 @@
 # earlyinit.
 
 import typing
+import argparse
 
-MYPY = False
-if MYPY:
-    # pylint: disable=unused-import,useless-suppression
+if typing.TYPE_CHECKING:
     from qutebrowser.utils import usertypes
     from qutebrowser.commands import command
 
@@ -35,9 +34,15 @@ class NoBackend:
 
     """Special object when there's no backend set so we notice that."""
 
+    @property
+    def name(self) -> str:
+        raise AssertionError("No backend set!")
+
     def __eq__(self, other: typing.Any) -> bool:
         raise AssertionError("No backend set!")
 
 
 backend = NoBackend()  # type: typing.Union[usertypes.Backend, NoBackend]
 commands = {}  # type: typing.Dict[str, command.Command]
+debug_flags = set()  # type: typing.Set[str]
+args = typing.cast(argparse.Namespace, None)
