@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2017 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -17,6 +17,8 @@
 # You should have received a copy of the GNU General Public License
 # along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
 
+# FIXME:typing Can we have less "# type: ignore" in here?
+
 """Misc. utilities related to Qt.
 
 Module attributes:
@@ -24,21 +26,28 @@ Module attributes:
              value.
     MINVALS: A dictionary of C/Qt types (as string) mapped to their minimum
              value.
+    MAX_WORLD_ID: The highest world ID allowed in this version of QtWebEngine.
 """
 
 
 import io
 import operator
 import contextlib
+import typing
 
 import pkg_resources
 from PyQt5.QtCore import (qVersion, QEventLoop, QDataStream, QByteArray,
                           QIODevice, QSaveFile, QT_VERSION_STR,
-                          PYQT_VERSION_STR)
+                          PYQT_VERSION_STR, QFileDevice, QObject)
+from PyQt5.QtGui import QColor
+from PyQt5.QtWidgets import QApplication
 try:
     from PyQt5.QtWebKit import qWebKitVersion
 except ImportError:  # pragma: no cover
-    qWebKitVersion = None
+    qWebKitVersion = None  # type: ignore  # noqa: N816
+
+from qutebrowser.misc import objects
+from qutebrowser.utils import usertypes
 
 
 MAXVALS = {
@@ -60,19 +69,22 @@ class QtOSError(OSError):
         qt_errno: The error attribute of the given QFileDevice, if applicable.
     """
 
-    def __init__(self, dev, msg=None):
+    def __init__(self, dev: QFileDevice, msg: str = None) -> None:
         if msg is None:
             msg = dev.errorString()
 
         super().__init__(msg)
 
+        self.qt_errno = None  # type: typing.Optional[QFileDevice.FileError]
         try:
             self.qt_errno = dev.error()
         except AttributeError:
-            self.qt_errno = None
+            pass
 
 
-def version_check(version, exact=False, compiled=True):
+def version_check(version: str,
+                  exact: bool = False,
+                  compiled: bool = True) -> bool:
     """Check if the Qt runtime version is the version supplied or newer.
 
     Args:
@@ -80,9 +92,6 @@ def version_check(version, exact=False, compiled=True):
         exact: if given, check with == instead of >=
         compiled: Set to False to not check the compiled version.
     """
-    # Catch code using the old API for this
-    assert exact not in [operator.gt, operator.lt, operator.ge, operator.le,
-                         operator.eq], exact
     if compiled and exact:
         raise ValueError("Can't use compiled=True with exact=True!")
 
@@ -98,14 +107,26 @@ def version_check(version, exact=False, compiled=True):
     return result
 
 
-def is_new_qtwebkit():
+# WORKAROUND for https://bugreports.qt.io/browse/QTBUG-69904
+MAX_WORLD_ID = 256 if version_check('5.11.2') else 11
+
+
+def is_new_qtwebkit() -> bool:
     """Check if the given version is a new QtWebKit."""
     assert qWebKitVersion is not None
     return (pkg_resources.parse_version(qWebKitVersion()) >
             pkg_resources.parse_version('538.1'))
 
 
-def check_overflow(arg, ctype, fatal=True):
+def is_single_process() -> bool:
+    """Check whether QtWebEngine is running in single-process mode."""
+    if objects.backend == usertypes.Backend.QtWebKit:
+        return False
+    args = QApplication.instance().arguments()
+    return '--single-process' in args
+
+
+def check_overflow(arg: int, ctype: str, fatal: bool = True) -> int:
     """Check if the given argument is in bounds for the given type.
 
     Args:
@@ -122,24 +143,22 @@ def check_overflow(arg, ctype, fatal=True):
     if arg > maxval:
         if fatal:
             raise OverflowError(arg)
-        else:
-            return maxval
+        return maxval
     elif arg < minval:
         if fatal:
             raise OverflowError(arg)
-        else:
-            return minval
+        return minval
     else:
         return arg
 
 
-def ensure_valid(obj):
+def ensure_valid(obj: QObject) -> None:
     """Ensure a Qt object with an .isValid() method is valid."""
     if not obj.isValid():
         raise QtValueError(obj)
 
 
-def check_qdatastream(stream):
+def check_qdatastream(stream: QDataStream) -> None:
     """Check the status of a QDataStream and raise OSError if it's not ok."""
     status_to_str = {
         QDataStream.Ok: "The data stream is operating normally.",
@@ -153,7 +172,7 @@ def check_qdatastream(stream):
         raise OSError(status_to_str[stream.status()])
 
 
-def serialize(obj):
+def serialize(obj: QObject) -> QByteArray:
     """Serialize an object into a QByteArray."""
     data = QByteArray()
     stream = QDataStream(data, QIODevice.WriteOnly)
@@ -161,20 +180,20 @@ def serialize(obj):
     return data
 
 
-def deserialize(data, obj):
+def deserialize(data: QByteArray, obj: QObject) -> None:
     """Deserialize an object from a QByteArray."""
     stream = QDataStream(data, QIODevice.ReadOnly)
     deserialize_stream(stream, obj)
 
 
-def serialize_stream(stream, obj):
+def serialize_stream(stream: QDataStream, obj: QObject) -> None:
     """Serialize an object into a QDataStream."""
     check_qdatastream(stream)
     stream << obj  # pylint: disable=pointless-statement
     check_qdatastream(stream)
 
 
-def deserialize_stream(stream, obj):
+def deserialize_stream(stream: QDataStream, obj: QObject) -> None:
     """Deserialize a QDataStream into an object."""
     check_qdatastream(stream)
     stream >> obj  # pylint: disable=pointless-statement
@@ -182,7 +201,11 @@ def deserialize_stream(stream, obj):
 
 
 @contextlib.contextmanager
-def savefile_open(filename, binary=False, encoding='utf-8'):
+def savefile_open(
+        filename: str,
+        binary: bool = False,
+        encoding: str = 'utf-8'
+) -> typing.Iterator[typing.Union['PyQIODevice', io.TextIOWrapper]]:
     """Context manager to easily use a QSaveFile."""
     f = QSaveFile(filename)
     cancelled = False
@@ -190,11 +213,16 @@ def savefile_open(filename, binary=False, encoding='utf-8'):
         open_ok = f.open(QIODevice.WriteOnly)
         if not open_ok:
             raise QtOSError(f)
+
         if binary:
-            new_f = PyQIODevice(f)
+            new_f = PyQIODevice(
+                f)  # type: typing.Union[PyQIODevice, io.TextIOWrapper]
         else:
-            new_f = io.TextIOWrapper(PyQIODevice(f), encoding=encoding)
+            new_f = io.TextIOWrapper(PyQIODevice(f),  # type: ignore
+                                     encoding=encoding)
+
         yield new_f
+
         new_f.flush()
     except:
         f.cancelWriting()
@@ -206,6 +234,13 @@ def savefile_open(filename, binary=False, encoding='utf-8'):
             raise QtOSError(f, msg="Commit failed!")
 
 
+def qcolor_to_qsscolor(c: QColor) -> str:
+    """Convert a QColor to a string that can be used in a QStyleSheet."""
+    ensure_valid(c)
+    return "rgba({}, {}, {}, {})".format(
+        c.red(), c.green(), c.blue(), c.alpha())
+
+
 class PyQIODevice(io.BufferedIOBase):
 
     """Wrapper for a QIODevice which provides a python interface.
@@ -214,34 +249,34 @@ class PyQIODevice(io.BufferedIOBase):
         dev: The underlying QIODevice.
     """
 
-    def __init__(self, dev):
+    def __init__(self, dev: QIODevice) -> None:
         super().__init__()
         self.dev = dev
 
-    def __len__(self):
+    def __len__(self) -> int:
         return self.dev.size()
 
-    def _check_open(self):
+    def _check_open(self) -> None:
         """Check if the device is open, raise ValueError if not."""
         if not self.dev.isOpen():
             raise ValueError("IO operation on closed device!")
 
-    def _check_random(self):
+    def _check_random(self) -> None:
         """Check if the device supports random access, raise OSError if not."""
         if not self.seekable():
             raise OSError("Random access not allowed!")
 
-    def _check_readable(self):
+    def _check_readable(self) -> None:
         """Check if the device is readable, raise OSError if not."""
         if not self.dev.isReadable():
             raise OSError("Trying to read unreadable file!")
 
-    def _check_writable(self):
+    def _check_writable(self) -> None:
         """Check if the device is writable, raise OSError if not."""
         if not self.writable():
             raise OSError("Trying to write to unwritable file!")
 
-    def open(self, mode):
+    def open(self, mode: QIODevice.OpenMode) -> contextlib.closing:
         """Open the underlying device and ensure opening succeeded.
 
         Raises OSError if opening failed.
@@ -258,14 +293,14 @@ class PyQIODevice(io.BufferedIOBase):
             raise QtOSError(self.dev)
         return contextlib.closing(self)
 
-    def close(self):
+    def close(self) -> None:
         """Close the underlying device."""
         self.dev.close()
 
-    def fileno(self):
+    def fileno(self) -> int:
         raise io.UnsupportedOperation
 
-    def seek(self, offset, whence=io.SEEK_SET):
+    def seek(self, offset: int, whence: int = io.SEEK_SET) -> int:
         self._check_open()
         self._check_random()
         if whence == io.SEEK_SET:
@@ -280,25 +315,27 @@ class PyQIODevice(io.BufferedIOBase):
         if not ok:
             raise QtOSError(self.dev, msg="seek failed!")
 
-    def truncate(self, size=None):  # pylint: disable=unused-argument
+        return self.dev.pos()
+
+    def truncate(self, size: int = None) -> int:
         raise io.UnsupportedOperation
 
     @property
-    def closed(self):
+    def closed(self) -> bool:
         return not self.dev.isOpen()
 
-    def flush(self):
+    def flush(self) -> None:
         self._check_open()
         self.dev.waitForBytesWritten(-1)
 
-    def isatty(self):
+    def isatty(self) -> bool:
         self._check_open()
         return False
 
-    def readable(self):
+    def readable(self) -> bool:
         return self.dev.isReadable()
 
-    def readline(self, size=-1):
+    def readline(self, size: int = -1) -> QByteArray:
         self._check_open()
         self._check_readable()
 
@@ -319,36 +356,39 @@ class PyQIODevice(io.BufferedIOBase):
 
         if buf is None:
             raise QtOSError(self.dev)
-        return buf
+        return buf  # type: ignore
 
-    def seekable(self):
+    def seekable(self) -> bool:
         return not self.dev.isSequential()
 
-    def tell(self):
+    def tell(self) -> int:
         self._check_open()
         self._check_random()
         return self.dev.pos()
 
-    def writable(self):
+    def writable(self) -> bool:
         return self.dev.isWritable()
 
-    def write(self, b):
+    def write(self, data: str) -> int:  # type: ignore
         self._check_open()
         self._check_writable()
-        num = self.dev.write(b)
-        if num == -1 or num < len(b):
+        num = self.dev.write(data)  # type: ignore
+        if num == -1 or num < len(data):
             raise QtOSError(self.dev)
         return num
 
-    def read(self, size=-1):
+    def read(self, size: typing.Optional[int] = None) -> QByteArray:
         self._check_open()
         self._check_readable()
-        if size < 0:
+
+        if size in [None, -1]:
             buf = self.dev.readAll()
         else:
-            buf = self.dev.read(size)
+            buf = self.dev.read(size)  # type: ignore
+
         if buf is None:
             raise QtOSError(self.dev)
+
         return buf
 
 
@@ -356,7 +396,7 @@ class QtValueError(ValueError):
 
     """Exception which gets raised by ensure_valid."""
 
-    def __init__(self, obj):
+    def __init__(self, obj: QObject) -> None:
         try:
             self.reason = obj.errorString()
         except AttributeError:
@@ -374,14 +414,18 @@ class EventLoop(QEventLoop):
     Raises an exception when doing exec_() multiple times.
     """
 
-    def __init__(self, parent=None):
+    def __init__(self, parent: QObject = None) -> None:
         super().__init__(parent)
         self._executing = False
 
-    def exec_(self, flags=QEventLoop.AllEvents):
+    def exec_(
+            self,
+            flags: QEventLoop.ProcessEventsFlag = QEventLoop.AllEvents
+    ) -> int:
         """Override exec_ to raise an exception when re-running."""
         if self._executing:
             raise AssertionError("Eventloop is already running!")
         self._executing = True
-        super().exec_(flags)
+        status = super().exec_(flags)  # type: ignore
         self._executing = False
+        return status
