@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2018-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2018-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -27,6 +27,7 @@ https://cs.chromium.org/chromium/src/extensions/common/url_pattern.h
 
 import ipaddress
 import fnmatch
+import typing
 import urllib.parse
 
 from PyQt5.QtCore import QUrl
@@ -48,6 +49,7 @@ class UrlPattern:
         _SCHEMES_WITHOUT_HOST: Schemes which don't need a host.
 
     Attributes:
+        host: The host to match to, or None for any host.
         _pattern: The given pattern as string.
         _match_all: Whether the pattern should match all URLs.
         _match_subdomains: Whether the pattern should match subdomains of the
@@ -56,7 +58,6 @@ class UrlPattern:
                  Note that with Chromium, '*'/None only matches http/https and
                  not file/ftp. We deviate from that as per-URL settings aren't
                  security relevant.
-        _host: The host to match to, or None for any host.
         _path: The path to match to, or None for any path.
         _port: The port to match to as integer, or None for any port.
     """
@@ -64,15 +65,15 @@ class UrlPattern:
     _DEFAULT_PORTS = {'https': 443, 'http': 80, 'ftp': 21}
     _SCHEMES_WITHOUT_HOST = ['about', 'file', 'data', 'javascript']
 
-    def __init__(self, pattern):
+    def __init__(self, pattern: str) -> None:
         # Make sure all attributes are initialized if we exit early.
         self._pattern = pattern
         self._match_all = False
-        self._match_subdomains = False
-        self._scheme = None
-        self._host = None
-        self._path = None
-        self._port = None
+        self._match_subdomains = False  # type: bool
+        self._scheme = None  # type: typing.Optional[str]
+        self.host = None  # type: typing.Optional[str]
+        self._path = None  # type: typing.Optional[str]
+        self._port = None  # type: typing.Optional[int]
 
         # > The special pattern <all_urls> matches any URL that starts with a
         # > permitted scheme.
@@ -99,27 +100,26 @@ class UrlPattern:
         self._init_path(parsed)
         self._init_port(parsed)
 
-    def _to_tuple(self):
+    def _to_tuple(self) -> typing.Tuple:
         """Get a pattern with information used for __eq__/__hash__."""
         return (self._match_all, self._match_subdomains, self._scheme,
-                self._host, self._path, self._port)
+                self.host, self._path, self._port)
 
-    def __hash__(self):
+    def __hash__(self) -> int:
         return hash(self._to_tuple())
 
-    def __eq__(self, other):
+    def __eq__(self, other: typing.Any) -> bool:
         if not isinstance(other, UrlPattern):
             return NotImplemented
-        # pylint: disable=protected-access
         return self._to_tuple() == other._to_tuple()
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return utils.get_repr(self, pattern=self._pattern, constructor=True)
 
-    def __str__(self):
+    def __str__(self) -> str:
         return self._pattern
 
-    def _fixup_pattern(self, pattern):
+    def _fixup_pattern(self, pattern: str) -> str:
         """Make sure the given pattern is parseable by urllib.parse."""
         if pattern.startswith('*:'):  # Any scheme, but *:// is unparseable
             pattern = 'any:' + pattern[2:]
@@ -136,7 +136,7 @@ class UrlPattern:
 
         return pattern
 
-    def _init_scheme(self, parsed):
+    def _init_scheme(self, parsed: urllib.parse.ParseResult) -> None:
         """Parse the scheme from the given URL.
 
         Deviation from Chromium:
@@ -151,7 +151,7 @@ class UrlPattern:
 
         self._scheme = parsed.scheme
 
-    def _init_path(self, parsed):
+    def _init_path(self, parsed: urllib.parse.ParseResult) -> None:
         """Parse the path from the given URL.
 
         Deviation from Chromium:
@@ -162,14 +162,14 @@ class UrlPattern:
 
         if parsed.path == '/*':
             self._path = None
-        elif parsed.path == '':
+        elif not parsed.path:
             # When the user doesn't add a trailing slash, we assume the pattern
             # matches any path.
             self._path = None
         else:
             self._path = parsed.path
 
-    def _init_host(self, parsed):
+    def _init_host(self, parsed: urllib.parse.ParseResult) -> None:
         """Parse the host from the given URL.
 
         Deviation from Chromium:
@@ -178,7 +178,7 @@ class UrlPattern:
         if parsed.hostname is None or not parsed.hostname.strip():
             if self._scheme not in self._SCHEMES_WITHOUT_HOST:
                 raise ParseError("Pattern without host")
-            assert self._host is None
+            assert self.host is None
             return
 
         if parsed.netloc.startswith('['):
@@ -187,7 +187,7 @@ class UrlPattern:
             url.setHost(parsed.hostname)
             if not url.isValid():
                 raise ParseError(url.errorString())
-            self._host = url.host()
+            self.host = url.host()
             return
 
         # FIXME what about multiple dots?
@@ -197,19 +197,19 @@ class UrlPattern:
             self._match_subdomains = True
 
         if not host_parts:
-            self._host = None
+            self.host = None
             return
 
-        self._host = '.'.join(host_parts)
+        self.host = '.'.join(host_parts)
 
-        if self._host.endswith('.*'):
+        if self.host.endswith('.*'):
             # Special case to have a nicer error
             raise ParseError("TLD wildcards are not implemented yet")
-        if '*' in self._host:
+        if '*' in self.host:
             # Only * or *.foo is allowed as host.
             raise ParseError("Invalid host wildcard")
 
-    def _init_port(self, parsed):
+    def _init_port(self, parsed: urllib.parse.ParseResult) -> None:
         """Parse the port from the given URL.
 
         Deviation from Chromium:
@@ -226,15 +226,16 @@ class UrlPattern:
             except ValueError as e:
                 raise ParseError("Invalid port: {}".format(e))
 
-        if (self._scheme not in list(self._DEFAULT_PORTS) + [None] and
-                self._port is not None):
+        scheme_has_port = (self._scheme in list(self._DEFAULT_PORTS) or
+                           self._scheme is None)
+        if self._port is not None and not scheme_has_port:
             raise ParseError("Ports are unsupported with {} scheme".format(
                 self._scheme))
 
-    def _matches_scheme(self, scheme):
+    def _matches_scheme(self, scheme: str) -> bool:
         return self._scheme is None or self._scheme == scheme
 
-    def _matches_host(self, host):
+    def _matches_host(self, host: str) -> bool:
         # FIXME what about multiple dots?
         host = host.rstrip('.')
 
@@ -244,11 +245,11 @@ class UrlPattern:
         # Contrary to Chromium, we don't need to check for
         # self._match_subdomains, as we want to return True here for e.g.
         # file:// as well.
-        if self._host is None:
+        if self.host is None:
             return True
 
         # If the hosts are exactly equal, we have a match.
-        if host == self._host:
+        if host == self.host:
             return True
 
         # Otherwise, we can only match if our match pattern matches subdomains.
@@ -261,20 +262,20 @@ class UrlPattern:
             return False
 
         # Check if the test host is a subdomain of our host.
-        if len(host) <= (len(self._host) + 1):
+        if len(host) <= (len(self.host) + 1):
             return False
 
-        if not host.endswith(self._host):
+        if not host.endswith(self.host):
             return False
 
-        return host[len(host) - len(self._host) - 1] == '.'
+        return host[len(host) - len(self.host) - 1] == '.'
 
-    def _matches_port(self, scheme, port):
+    def _matches_port(self, scheme: str, port: int) -> bool:
         if port == -1 and scheme in self._DEFAULT_PORTS:
             port = self._DEFAULT_PORTS[scheme]
         return self._port is None or self._port == port
 
-    def _matches_path(self, path):
+    def _matches_path(self, path: str) -> bool:
         if self._path is None:
             return True
 
@@ -286,7 +287,7 @@ class UrlPattern:
         # doesn't rely on regexes. Do we need that too?
         return fnmatch.fnmatchcase(path, self._path)
 
-    def matches(self, qurl):
+    def matches(self, qurl: QUrl) -> bool:
         """Check if the pattern matches the given QUrl."""
         qtutils.ensure_valid(qurl)
 
