@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2017 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -17,7 +17,7 @@
 # You should have received a copy of the GNU General Public License
 # along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
 
-"""Utilities to show various version informations."""
+"""Utilities to show various version information."""
 
 import re
 import sys
@@ -29,6 +29,8 @@ import importlib
 import collections
 import enum
 import datetime
+import getpass
+import typing
 
 import attr
 import pkg_resources
@@ -41,17 +43,18 @@ from PyQt5.QtWidgets import QApplication
 try:
     from PyQt5.QtWebKit import qWebKitVersion
 except ImportError:  # pragma: no cover
-    qWebKitVersion = None
-
-try:
-    from PyQt5.QtWebEngineWidgets import QWebEngineProfile
-except ImportError:  # pragma: no cover
-    QWebEngineProfile = None
+    qWebKitVersion = None  # type: ignore  # noqa: N816
 
 import qutebrowser
-from qutebrowser.utils import log, utils, standarddir, usertypes
-from qutebrowser.misc import objects, earlyinit, sql
+from qutebrowser.utils import log, utils, standarddir, usertypes, message
+from qutebrowser.misc import objects, earlyinit, sql, httpclient, pastebin
 from qutebrowser.browser import pdfjs
+from qutebrowser.config import config
+
+try:
+    from qutebrowser.browser.webengine import webenginesettings
+except ImportError:  # pragma: no cover
+    webenginesettings = None  # type: ignore
 
 
 @attr.s
@@ -59,18 +62,20 @@ class DistributionInfo:
 
     """Information about the running distribution."""
 
-    id = attr.ib()
-    parsed = attr.ib()
-    version = attr.ib()
-    pretty = attr.ib()
+    id = attr.ib()  # type: typing.Optional[str]
+    parsed = attr.ib()  # type: Distribution
+    version = attr.ib()  # type: typing.Optional[typing.Tuple[str, ...]]
+    pretty = attr.ib()  # type: str
 
 
+pastebin_url = None
 Distribution = enum.Enum(
     'Distribution', ['unknown', 'ubuntu', 'debian', 'void', 'arch',
-                     'gentoo', 'fedora', 'opensuse', 'linuxmint', 'manjaro'])
+                     'gentoo', 'fedora', 'opensuse', 'linuxmint', 'manjaro',
+                     'kde_flatpak'])
 
 
-def distribution():
+def distribution() -> typing.Optional[DistributionInfo]:
     """Get some information about the running Linux distribution.
 
     Returns:
@@ -92,29 +97,44 @@ def distribution():
     except (OSError, UnicodeDecodeError):
         return None
 
-    pretty = info.get('PRETTY_NAME', 'Unknown')
-    if pretty == 'Linux':  # Thanks, Funtoo
-        pretty = info.get('NAME', pretty)
+    pretty = info.get('PRETTY_NAME', None)
+    if pretty in ['Linux', None]:  # Funtoo has PRETTY_NAME=Linux
+        pretty = info.get('NAME', 'Unknown')
+    assert pretty is not None
 
     if 'VERSION_ID' in info:
-        dist_version = pkg_resources.parse_version(info['VERSION_ID'])
+        dist_version = pkg_resources.parse_version(
+            info['VERSION_ID']
+        )  # type: typing.Optional[typing.Tuple[str, ...]]
     else:
         dist_version = None
 
     dist_id = info.get('ID', None)
     id_mappings = {
         'funtoo': 'gentoo',  # does not have ID_LIKE=gentoo
+        'org.kde.Platform': 'kde_flatpak',
     }
-    try:
-        parsed = Distribution[id_mappings.get(dist_id, dist_id)]
-    except KeyError:
-        parsed = Distribution.unknown
+
+    parsed = Distribution.unknown
+    if dist_id is not None:
+        try:
+            parsed = Distribution[id_mappings.get(dist_id, dist_id)]
+        except KeyError:
+            pass
 
     return DistributionInfo(parsed=parsed, version=dist_version, pretty=pretty,
                             id=dist_id)
 
 
-def _git_str():
+def is_sandboxed() -> bool:
+    """Whether the environment has restricted access to the host system."""
+    current_distro = distribution()
+    if current_distro is None:
+        return False
+    return current_distro.parsed == Distribution.kde_flatpak
+
+
+def _git_str() -> typing.Optional[str]:
     """Try to find out git version.
 
     Return:
@@ -140,7 +160,7 @@ def _git_str():
         return None
 
 
-def _git_str_subprocess(gitpath):
+def _git_str_subprocess(gitpath: str) -> typing.Optional[str]:
     """Try to get the git commit ID and timestamp by calling git.
 
     Args:
@@ -166,8 +186,8 @@ def _git_str_subprocess(gitpath):
         return None
 
 
-def _release_info():
-    """Try to gather distribution release informations.
+def _release_info() -> typing.Sequence[typing.Tuple[str, str]]:
+    """Try to gather distribution release information.
 
     Return:
         list of (filename, content) tuples.
@@ -190,7 +210,7 @@ def _release_info():
     return data
 
 
-def _module_versions():
+def _module_versions() -> typing.Sequence[str]:
     """Get versions of optional modules.
 
     Return:
@@ -207,8 +227,9 @@ def _module_versions():
         ('cssutils', ['__version__']),
         ('attr', ['__version__']),
         ('PyQt5.QtWebEngineWidgets', []),
+        ('PyQt5.QtWebEngine', ['PYQT_WEBENGINE_VERSION_STR']),
         ('PyQt5.QtWebKitWidgets', []),
-    ])
+    ])  # type: typing.Mapping[str, typing.Sequence[str]]
     for modname, attributes in modules.items():
         try:
             module = importlib.import_module(modname)
@@ -228,7 +249,7 @@ def _module_versions():
     return lines
 
 
-def _path_info():
+def _path_info() -> typing.Mapping[str, str]:
     """Get info about important path names.
 
     Return:
@@ -247,7 +268,7 @@ def _path_info():
     return info
 
 
-def _os_info():
+def _os_info() -> typing.Sequence[str]:
     """Get operating system info.
 
     Return:
@@ -261,12 +282,14 @@ def _os_info():
     elif utils.is_windows:
         osver = ', '.join(platform.win32_ver())
     elif utils.is_mac:
-        release, versioninfo, machine = platform.mac_ver()
-        if all(not e for e in versioninfo):
+        release, info_tpl, machine = platform.mac_ver()
+        if all(not e for e in info_tpl):
             versioninfo = ''
         else:
-            versioninfo = '.'.join(versioninfo)
+            versioninfo = '.'.join(info_tpl)
         osver = ', '.join([e for e in [release, versioninfo, machine] if e])
+    elif utils.is_posix:
+        osver = ' '.join(platform.uname())
     else:
         osver = '?'
     lines.append('OS Version: {}'.format(osver))
@@ -276,7 +299,7 @@ def _os_info():
     return lines
 
 
-def _pdfjs_version():
+def _pdfjs_version() -> str:
     """Get the pdf.js version.
 
     Return:
@@ -302,21 +325,61 @@ def _pdfjs_version():
         return '{} ({})'.format(pdfjs_version, file_path)
 
 
-def _chromium_version():
-    """Get the Chromium version for QtWebEngine."""
-    if QWebEngineProfile is None:
-        # This should never happen
-        return 'unavailable'
-    profile = QWebEngineProfile()
-    ua = profile.httpUserAgent()
-    match = re.search(r' Chrome/([^ ]*) ', ua)
-    if not match:
-        log.misc.error("Could not get Chromium version from: {}".format(ua))
-        return 'unknown'
-    return match.group(1)
+def _chromium_version() -> str:
+    """Get the Chromium version for QtWebEngine.
+
+    This can also be checked by looking at this file with the right Qt tag:
+    http://code.qt.io/cgit/qt/qtwebengine.git/tree/tools/scripts/version_resolver.py#n41
+
+    Quick reference:
+
+    Qt 5.7:  Chromium 49
+             49.0.2623.111 (2016-03-31)
+             5.7.1: Security fixes up to 54.0.2840.87 (2016-11-01)
+
+    Qt 5.8:  Chromium 53
+             53.0.2785.148 (2016-08-31)
+             5.8.0: Security fixes up to 55.0.2883.75 (2016-12-01)
+
+    Qt 5.9:  Chromium 56
+    (LTS)    56.0.2924.122 (2017-01-25)
+             5.9.8: Security fixes up to 72.0.3626.121 (2019-03-01)
+
+    Qt 5.10: Chromium 61
+             61.0.3163.140 (2017-09-05)
+             5.10.1: Security fixes up to 64.0.3282.140 (2018-02-01)
+
+    Qt 5.11: Chromium 65
+             65.0.3325.151 (.1: .230) (2018-03-06)
+             5.11.3: Security fixes up to 70.0.3538.102 (2018-11-09)
+
+    Qt 5.12: Chromium 69
+    (LTS)    69.0.3497.113 (2018-09-27)
+             5.12.6: Security fixes up to 76.0.3809.87 (2019-07-30)
+                     plus fix for CVE-2019-13720 from Chrome 78
+
+    Qt 5.13: Chromium 73
+             73.0.3683.105 (~2019-02-28)
+             5.13.2: Security fixes up to 77.0.3865.120 (2019-10-10)
+
+    Qt 5.14: Chromium 77
+             77.0.3865.129 (~2019-10-10)
+             5.14.1: Security fixes up to 79.0.3945.117 (2020-01-07)
+
+    Also see https://www.chromium.org/developers/calendar
+    and https://chromereleases.googleblog.com/
+    """
+    if webenginesettings is None:
+        return 'unavailable'  # type: ignore
+
+    if webenginesettings.parsed_user_agent is None:
+        webenginesettings.init_user_agent()
+        assert webenginesettings.parsed_user_agent is not None
+
+    return webenginesettings.parsed_user_agent.upstream_browser_version
 
 
-def _backend():
+def _backend() -> str:
     """Get the backend line with relevant information."""
     if objects.backend == usertypes.Backend.QtWebKit:
         return 'new QtWebKit (WebKit {})'.format(qWebKitVersion())
@@ -334,20 +397,31 @@ def _uptime() -> datetime.timedelta:
     return time_delta
 
 
-def version():
-    """Return a string with various version informations."""
+def _autoconfig_loaded() -> str:
+    return "yes" if config.instance.yaml_loaded else "no"
+
+
+def _config_py_loaded() -> str:
+    if config.instance.config_py_loaded:
+        return "{} has been loaded".format(standarddir.config_py())
+    else:
+        return "no config.py was loaded"
+
+
+def version() -> str:
+    """Return a string with various version information."""
     lines = ["qutebrowser v{}".format(qutebrowser.__version__)]
     gitver = _git_str()
     if gitver is not None:
         lines.append("Git commit: {}".format(gitver))
 
-    lines.append("Backend: {}".format(_backend()))
+    lines.append('Backend: {}'.format(_backend()))
+    lines.append('Qt: {}'.format(earlyinit.qt_version()))
 
     lines += [
         '',
         '{}: {}'.format(platform.python_implementation(),
                         platform.python_version()),
-        'Qt: {}'.format(earlyinit.qt_version()),
         'PyQt: {}'.format(PYQT_VERSION_STR),
         '',
     ]
@@ -381,6 +455,7 @@ def version():
     lines += [
         'Frozen: {}'.format(hasattr(sys, 'frozen')),
         "Imported from {}".format(importpath),
+        "Using Python from {}".format(sys.executable),
         "Qt library executable path: {}, data path: {}".format(
             QLibraryInfo.location(QLibraryInfo.LibraryExecutablesPath),
             QLibraryInfo.location(QLibraryInfo.DataPath)
@@ -399,13 +474,15 @@ def version():
 
     lines += [
         '',
-        'Uptime: {}'.format(_uptime()),
+        'Autoconfig loaded: {}'.format(_autoconfig_loaded()),
+        'Config.py: {}'.format(_config_py_loaded()),
+        'Uptime: {}'.format(_uptime())
     ]
 
     return '\n'.join(lines)
 
 
-def opengl_vendor():  # pragma: no cover
+def opengl_vendor() -> typing.Optional[str]:  # pragma: no cover
     """Get the OpenGL vendor used.
 
     This returns a string such as 'nouveau' or
@@ -414,7 +491,13 @@ def opengl_vendor():  # pragma: no cover
     """
     assert QApplication.instance()
 
-    old_context = QOpenGLContext.currentContext()
+    override = os.environ.get('QUTE_FAKE_OPENGL_VENDOR')
+    if override is not None:
+        log.init.debug("Using override {}".format(override))
+        return override
+
+    old_context = typing.cast(typing.Optional[QOpenGLContext],
+                              QOpenGLContext.currentContext())
     old_surface = None if old_context is None else old_context.surface()
 
     surface = QOffscreenSurface()
@@ -423,12 +506,12 @@ def opengl_vendor():  # pragma: no cover
     ctx = QOpenGLContext()
     ok = ctx.create()
     if not ok:
-        log.init.debug("opengl_vendor: Creating context failed!")
+        log.init.debug("Creating context failed!")
         return None
 
     ok = ctx.makeCurrent(surface)
     if not ok:
-        log.init.debug("opengl_vendor: Making context current failed!")
+        log.init.debug("Making context current failed!")
         return None
 
     try:
@@ -439,9 +522,14 @@ def opengl_vendor():  # pragma: no cover
         vp = QOpenGLVersionProfile()
         vp.setVersion(2, 0)
 
-        vf = ctx.versionFunctions(vp)
+        try:
+            vf = ctx.versionFunctions(vp)
+        except ImportError as e:
+            log.init.debug("Importing version functions failed: {}".format(e))
+            return None
+
         if vf is None:
-            log.init.debug("opengl_vendor: Getting version functions failed!")
+            log.init.debug("Getting version functions failed!")
             return None
 
         return vf.glGetString(vf.GL_VENDOR)
@@ -449,3 +537,43 @@ def opengl_vendor():  # pragma: no cover
         ctx.doneCurrent()
         if old_context and old_surface:
             old_context.makeCurrent(old_surface)
+
+
+def pastebin_version(pbclient: pastebin.PastebinClient = None) -> None:
+    """Pastebin the version and log the url to messages."""
+    def _yank_url(url: str) -> None:
+        utils.set_clipboard(url)
+        message.info("Version url {} yanked to clipboard.".format(url))
+
+    def _on_paste_version_success(url: str) -> None:
+        assert pbclient is not None
+        global pastebin_url
+        url = url.strip()
+        _yank_url(url)
+        pbclient.deleteLater()
+        pastebin_url = url
+
+    def _on_paste_version_err(text: str) -> None:
+        assert pbclient is not None
+        message.error("Failed to pastebin version"
+                      " info: {}".format(text))
+        pbclient.deleteLater()
+
+    if pastebin_url:
+        _yank_url(pastebin_url)
+        return
+
+    app = QApplication.instance()
+    http_client = httpclient.HTTPClient()
+
+    misc_api = pastebin.PastebinClient.MISC_API_URL
+    pbclient = pbclient or pastebin.PastebinClient(http_client, parent=app,
+                                                   api_url=misc_api)
+
+    pbclient.success.connect(_on_paste_version_success)
+    pbclient.error.connect(_on_paste_version_err)
+
+    pbclient.paste(getpass.getuser(),
+                   "qute version info {}".format(qutebrowser.__version__),
+                   version(),
+                   private=True)
