@@ -1,8 +1,8 @@
 #!/usr/bin/env python3
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2017 Claude (longneck) <longneck@scratchbook.ch>
-# Copyright 2014-2017 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2018 Claude (longneck) <longneck@scratchbook.ch>
 
 # This file is part of qutebrowser.
 #
@@ -333,6 +333,8 @@ def import_chrome(profile, bookmark_types, output_format):
 
         def bm_tree_walk(bm, template):
             """Recursive function to walk through bookmarks."""
+            if not isinstance(bm, dict):
+                return
             assert 'type' in bm, bm
             if bm['type'] == 'url':
                 if urllib.parse.urlparse(bm['url']).scheme != 'chrome':
